/****************************************************************************
*
* FOOTPRNT.C
*
*    VERSION : 00.00
*    CREATED : 13/05/98
*   MODIFIED : 13/05/98
*	   TABS : 5
*
* DESCRIPTION
*	Triggers and updates footprint effects
*
*****************************************************************************/

/*---------------------------------------------------------------------------
 * Import Headers
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
#include <sys\types.h>
#include <libgte.h>
#include <libgpu.h>
#include <gtemac.h>
#include <inline_c.h>
#include "..\spec_psx\typedefs.h"
#include "..\spec_psx\maths.h"
#include "..\spec_psx\gpu.h"
#else
#include "../specific/stypes.h"
#endif

#include "objects.h"
#include "items.h"
#include "control.h"
#include "sound.h"
#include "effect2.h"
#include "lara.h"
#include "footprnt.h"

/*---------------------------------------------------------------------------
 * Constants
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
#define CLIPL	(-32)
#define CLIPR	(MY_PSX_W+32)
#define CLIPT	(-32)
#define CLIPB	(MY_PSX_H-32)
#endif

/*---------------------------------------------------------------------------
 * Externals
\*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
 * Globals
\*--------------------------------------------------------------------------*/

int FootPrintNum;
FOOTPRINT FootPrint[MAX_FOOTPRINTS];

/*---------------------------------------------------------------------------
 * Locals
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
static SVECTOR FV1 = {    0, 0, -64 };
static SVECTOR FV2 = { -128, 0,  64 };
static SVECTOR FV3 = {  128, 0,  64 };
#endif


/*---------------------------------------------------------------------------
 * Macros
\*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
 * Local Functions
\*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
 * Public Functions
\*--------------------------------------------------------------------------*/

/****************************************************************************
* InitFootPrints
*
* SYNOPSIS
*	void InitFootPrints()
*
* PURPOSE
*	Initialises the footprint structures
*
* INPUTS
*	None
*
* RETURNS
*	Nothing
*/
void InitFootPrints()
{
	int i;

	for ( i=0 ; i<MAX_FOOTPRINTS ; i++ )
		FootPrint[i].Active = 0;

	FootPrintNum = 0;
}

/****************************************************************************
* AddFootprint
*
* SYNOPSIS
*	void AddFootprint(ITEM_INFO *item)
*
* PURPOSE
*	Creates a footprint effect
*
* INPUTS
*	item		= ptr to current item
*
* RETURNS
*	Nothing
*/
void AddFootprint(ITEM_INFO *item)
{
	sint16 room_num;
	FLOOR_INFO *floor;
	PHD_VECTOR pos = { 0, 0, 0 };
	static char Sounds[] = {
		0,	// mud
		5,	// snow
		3,	// sand
		2,	// gravel
		1,	// ice
		9,	// water
		9,	// stone
		4,	// wood
		6,	// metal
		9,	// marble
		3,	// grass
		9,	// concrete
		4,	// old wood
		6	// old metal
	 };

/* -------- translate to required foot */

	if (FXType == SFX_LANDONLY)
		GetLaraHandAbsPosition(&pos, LEFT_FOOT);
	else
		GetLaraHandAbsPosition(&pos, RIGHT_FOOT);

/* -------- determine whether texture underfoot can have a footprint */

	room_num = item->room_number;
	floor = GetFloor(pos.x, pos.y, pos.z, &room_num);

	if ((floor->fx != 6)
	&& (floor->fx != 5)
	&&  (floor->fx != 9)
	&&  (floor->fx != 11))
		SoundEffect(288+Sounds[floor->fx], &lara_item->pos, 0);

	if ((floor->fx <= 4) && (!OnObject))
	{
	/* -------- initialise footprint */

		FOOTPRINT *f;

		f = &FootPrint[FootPrintNum];

		f->x = pos.x;
		f->y = GetHeight(floor, pos.x, pos.y, pos.z);
		f->z = pos.z;
		f->YRot = item->pos.y_rot;
		f->Active = 0x200;

		FootPrintNum = (FootPrintNum + 1) & (MAX_FOOTPRINTS-1);
	}
}

/****************************************************************************
* S_DrawFootPrints
*
* SYNOPSIS
*	void S_DrawFootPrints()
*
* PURPOSE
*	Draws all active footprints
*
* INPUTS
*	None
*
* RETURNS
*	Nothing
*/

#ifdef PSX_VERSION

void S_DrawFootPrints()
{
	int i;
	FOOTPRINT *f;
	POLY_FT3 *ft3;
	unsigned char u1, u2, v1, v2;
	PSXSPRITESTRUCT *pSpriteInfo;

	f = &FootPrint[0];

	pSpriteInfo=psxspriteinfo+objects[EXPLOSION1].mesh_index+17;
	u1 = pSpriteInfo->u1;
	u2 = pSpriteInfo->u2;
	v1 = pSpriteInfo->v1;
	v2 = pSpriteInfo->v2;

	ft3=(POLY_FT3*)db.polyptr;

	for ( i=0 ; i<MAX_FOOTPRINTS ; i++,f++ )
	{
		if (f->Active)
		{
			long z;
			sint16 rgb;

			if ((char *)ft3 >= db.polybuf_limit)
			{
				db.polyptr = (char *)ft3;
				return;
			}

			f->Active--;

			if (f->Active > 28)
				rgb = 112;
			else
				rgb = f->Active<<2;

			mPushMatrix();
			mTranslateAbsXYZ(f->x, f->y, f->z);
			mRotY(f->YRot);

			gte_ldv3(&FV1, &FV2, &FV3);
			gte_rtpt();

			setlen(ft3, 7);
			setcode(ft3, 0x26);
			setRGB0(ft3, rgb, rgb, rgb);
			setUV3(ft3, u1, v1, u2, v1, u1, v2);
			ft3->clut = pSpriteInfo->clut;
			ft3->tpage = (pSpriteInfo->tpage & (~96)) | (2<<5);

			gte_stsxy3_ft3(ft3);
			gte_avsz3();
			mPopMatrix();
			gte_stotz(&z);

			if ((((ft3->x0 < CLIPL) || (ft3->x0 > CLIPR))
			&&   ((ft3->x1 < CLIPL) || (ft3->x1 > CLIPR))
			&&   ((ft3->x2 < CLIPL) || (ft3->x2 > CLIPR))
			&&   ((ft3->y0 < CLIPT) || (ft3->y0 > CLIPB))
			&&   ((ft3->y1 < CLIPT) || (ft3->y1 > CLIPB))
			&&   ((ft3->y2 < CLIPT) || (ft3->y2 > CLIPB)))
			||   ((z < 2) || (z > 2560)))
				continue;

			MyaddPrim(db.ot, z-2, ft3);
			ft3++;
		}
	}

	db.polyptr = (uchar *)ft3;
}

#endif
