/***********************************************************************************************************/
/*                                                                                                         */
/* Game Saving Stuff                                                                                    	  */
/*                                                                                                         */
/***********************************************************************************************************/

/*---------------------------------------------------------------------------
 *	Import Headers
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
#include <sys\types.h>
#include <stdio.h>
#include "../spec_psx/typedefs.h"
#include "../spec_psx/memory.h"
#include "../spec_psx/memcard.h"
#include "../spec_psx/specific.h"
extern sint16 *mesh_base;
#else
#include "../specific/standard.h"
#include "../specific/global.h"
#include "../specific/output.h"
#include "../specific/input.h"
#include "../game/inventry.h"
#endif

#include "objects.h"
#include "lara.h"
#include "laraanim.h"
#include "camera.h"
#include "control.h"
#include "savegame.h"
#include "invfunc.h"
#include "laramisc.h"
#include "boat.h"
#include "pickup.h"
#include "moveblok.h"
#include "skidoo.h"
#include "traps.h"
#include "lot.h"
#include "title.h"
#include "biggun.h"
#include "boatinfo.h"
#include "kayak.h"
#include "minecart.h"
#include "quadbike.h"
#include "sub.h"
#include "fish.h"

extern SG_COL	req_sel_gour2[];  // Outline Gouraud

extern char TribeBossShieldOn;
extern ITEM_INFO *TribeBossItem;
extern char lizard_man_active;

#define SAVE_CREATURE_SIZE (sizeof(CREATURE_INFO)-sizeof(LOT_INFO)-sizeof(sint32)-sizeof(PHD_VECTOR)-sizeof(sint16))

/**************************************** GLOBAL VARIABLES ****************************************************/

SAVEGAME_INFO savegame;

/* For dealing with data in save game buffer */
int SGcount;
char *SGpoint;

int number_cameras=0;

#define SAVE_CREATURE (1<<31)

/***************************************** FUNCTION CODE ******************************************************/

int LoadSaveGameInfo(void)
{
#ifdef PSX_VERSION
	/* Try to load a savegame packet. Returns 1 if success, 0 if failure */
/*	int	status;
	SAVEGAME_INFO *temp;

	temp = (SAVEGAME_INFO *)game_malloc(sizeof(SAVEGAME_INFO)+127, TEMP_ALLOC);

	if (!S_LoadGame(temp, sizeof(SAVEGAME_INFO), 0))
	{
		game_free(sizeof(SAVEGAME_INFO)+127, TEMP_ALLOC);
		return (0);
	}
*/
	/* Copy temp packet into genuine packet */
//	S_MemCpy(&savegame, temp, sizeof(SAVEGAME_INFO));
//	game_free(sizeof(SAVEGAME_INFO)+127, TEMP_ALLOC);

	return(S_LoadGame(&savegame, sizeof(SAVEGAME_INFO)-128, 0));
#else
	return (1);
#endif
}


void InitialiseStartInfo(void)
{
	int i;

#if defined(GAMEDEBUG) && defined(PSX_VERSION)
	printf("InitialiseStartInfo()\n");
#endif
	/* Initialise start data for each level */
#ifdef PC_VERSION
	if (savegame.bonus_flag == 0)
	{
#endif
		for ( i=0;i<MAXIMUM_LEVELS;i++ )
		{
			ModifyStartInfo(i);
			savegame.start[i].available = 0;
			savegame.start[i].timer = 0;
			savegame.start[i].ammo_used = 0;
			savegame.start[i].ammo_hit = 0;
			savegame.start[i].distance_travelled = 0;
			savegame.start[i].kills = 0;
			savegame.start[i].secrets_found = 0;
			savegame.start[i].health_used = 0;
		}

		for (i=0;i<10;i++)
		{
			savegame.best_assault_times[i]=0;
			savegame.best_quadbike_times[i]=0;
		}

		savegame.AfterAdventureSave = 0;
		savegame.WorldRequired = 0;
		savegame.IndiaComplete = 0;
		savegame.SPacificComplete = 0;
		savegame.NevadaComplete = 0;
		savegame.LondonComplete = 0;
		savegame.AntarcticaComplete = 0;
		savegame.PeruComplete = 0;
		savegame.AfterIndia = 0;
		savegame.AfterSPacific = 0;
		savegame.AfterLondon = 0;
		savegame.AfterNevada = 0;

		savegame.QuadbikeKeyFlag = 0;

		/* Player can start on gym or level1 */
		savegame.start[LV_GYM].available = 1;
		savegame.start[LV_FIRSTLEVEL].available = 1;
		savegame.bonus_flag = 0;
		savegame.GameComplete = 0;
#ifdef PC_VERSION
	}
#endif
}


void ModifyStartInfo(int level_number)
{
	START_INFO *start;

	start = &savegame.start[level_number];

	start->got_pistols = 1;      			// Always got Pistols
	start->gun_type = LG_PISTOLS;           // assume they are readied
	start->pistol_ammo = 1000;              // Inifinite ammo

	switch (level_number)
	{
		case LV_GYM:
			start->available = 1;
			start->pistol_ammo = start->magnum_ammo = start->uzi_ammo = start->shotgun_ammo = start->harpoon_ammo = start->rocket_ammo = start->grenade_ammo = start->m16_ammo = start->harpoon_ammo = start->rocket_ammo = 0;
			start->got_pistols = start->got_magnums = start->got_uzis = start->got_shotgun = start->got_harpoon = start->got_rocket = start->got_grenade = start->got_m16 = start->got_harpoon = 0;
			start->num_medis = start->num_big_medis = start->num_flares = 0;
			start->num_sgcrystals = 1;
			start->gun_type = LG_UNARMED;
			start->gun_status = LG_ARMLESS;
			break;

		case LV_FIRSTLEVEL:
			start->available = 1;
			start->num_flares = 2;
			start->num_medis = 1;
			start->num_big_medis = 1;
			start->num_sgcrystals = 1;
			start->got_magnums = start->got_uzis = start->got_m16 = start->got_harpoon = start->got_rocket = start->got_grenade = 0;
			start->magnum_ammo = start->uzi_ammo = start->m16_ammo = start->harpoon_ammo = start->rocket_ammo = start->grenade_ammo = 0;
			start->gun_status = LG_ARMLESS;
			break;

	}

	if ( savegame.bonus_flag && level_number!=LV_GYM )
	{
		start->available = 1;
		start->got_pistols = start->got_magnums = start->got_uzis = start->got_shotgun = start->got_rocket = start->got_grenade = start->got_m16 = start->got_harpoon = 1;
		start->magnum_ammo = start->uzi_ammo = start->shotgun_ammo = start->rocket_ammo = start->grenade_ammo = start->m16_ammo = start->harpoon_ammo = 10000;
		start->gun_type = LG_ROCKET;
		start->num_flares = 255;
	}
}


void CreateStartInfo(int level_number)
{
	/* Create the start info for a level */
	START_INFO *start;

	start = &savegame.start[level_number];
	start->available = 1;

	/* Insert Pistol Stuff */
	if (Inv_RequestItem(GUN_ITEM))
	{
		start->pistol_ammo = 1000;
		start->got_pistols = 1;
	}
	else
	{
		start->pistol_ammo = 1000;
		start->got_pistols = 0;
	}

	/* Insert Magnum Stuff */
	if (Inv_RequestItem(MAGNUM_ITEM))
	{
		start->magnum_ammo = lara.magnums.ammo;
		start->got_magnums = 1;
	}
	else
	{
		start->magnum_ammo = Inv_RequestItem(MAG_AMMO_ITEM)*MAGNUM_AMMO_QTY;
		start->got_magnums = 0;
	}

	/* Insert Uzi Stuff */
	if (Inv_RequestItem(UZI_ITEM))
	{
		start->uzi_ammo = lara.uzis.ammo;
		start->got_uzis = 1;
	}
	else
	{
		start->uzi_ammo = Inv_RequestItem(UZI_AMMO_ITEM)*UZI_AMMO_QTY;
		start->got_uzis = 0;
	}

	/* Insert Shotgun Stuff */
	if (Inv_RequestItem(SHOTGUN_ITEM))
	{
		start->shotgun_ammo = lara.shotgun.ammo;
		start->got_shotgun = 1;
	}
	else
	{
		start->shotgun_ammo = Inv_RequestItem(SG_AMMO_ITEM)*SHOTGUN_AMMO_QTY;
		start->got_shotgun = 0;
	}

	/* Insert Harpoon Stuff */
	if (Inv_RequestItem(HARPOON_ITEM))
	{
		start->harpoon_ammo = lara.harpoon.ammo;
		start->got_harpoon = 1;
	}
	else
	{
		start->harpoon_ammo = lara.harpoon.ammo;
//		start->harpoon_ammo = Inv_RequestItem(HARPOON_AMMO_ITEM)*HARPOON_AMMO_QTY;
		start->got_harpoon = 0;
	}

	/* Insert M16 Stuff */
	if (Inv_RequestItem(M16_ITEM))
	{
		start->m16_ammo = lara.m16.ammo;
		start->got_m16 = 1;
	}
	else
	{
		start->m16_ammo = Inv_RequestItem(M16_AMMO_ITEM)*M16_AMMO_QTY;
		start->got_m16 = 0;
	}

	/* Insert Rocket Stuff */
	if (Inv_RequestItem(ROCKET_GUN_ITEM))
	{
		start->rocket_ammo = lara.rocket.ammo;
		start->got_rocket = 1;
	}
	else
	{
		start->rocket_ammo = Inv_RequestItem(ROCKET_AMMO_ITEM)*ROCKET_AMMO_QTY;
		start->got_rocket = 0;
	}

	/* Insert Grenade Stuff */
	if (Inv_RequestItem(GRENADE_GUN_ITEM))
	{
		start->grenade_ammo = lara.grenade.ammo;
		start->got_grenade = 1;
	}
	else
	{
		start->grenade_ammo = Inv_RequestItem(GRENADE_AMMO_ITEM)*GRENADE_AMMO_QTY;
		start->got_grenade = 0;
	}

	start->num_flares = Inv_RequestItem(FLARE_ITEM);
	start->num_medis = Inv_RequestItem(MEDI_ITEM);
	start->num_big_medis = Inv_RequestItem(BIGMEDI_ITEM);
	start->num_sgcrystals = Inv_RequestItem(SAVEGAME_CRYSTAL_ITEM);

	start->num_icon1 = Inv_RequestItem(ICON_PICKUP1_ITEM);
	start->num_icon2 = Inv_RequestItem(ICON_PICKUP2_ITEM);
	start->num_icon3 = Inv_RequestItem(ICON_PICKUP3_ITEM);
	start->num_icon4 = Inv_RequestItem(ICON_PICKUP4_ITEM);

	if (lara.gun_type == LG_FLARE)
		start->gun_type = (char)lara.last_gun_type;
	else
		start->gun_type = (char)lara.gun_type;
	start->gun_status = LG_ARMLESS;
}


void	CreateSaveGameInfo()
{
	int i,texplode;
	char tflag;
	ITEM_INFO *item;
	OBJECT_INFO *obj;
	char	noresetexplode=0;

#ifdef FULL_SAVE
//##	CREATURE_INFO *creature;
	int number_flares, flare_age;
#endif

	savegame.current_level = (sint16)CurrentLevel;

#ifdef PSX_VERSION
	if (savegame.mid_level_save)
		CreateStartInfo(MAXIMUM_LEVELS-2);	// ## was 0 but now we save Gym
	else
#endif
		CreateStartInfo(CurrentLevel);

/* Store information about Lara's inventory over what she has at start */
	savegame.num_pickup1 = Inv_RequestItem(PICKUP_ITEM1);
	savegame.num_pickup2 = Inv_RequestItem(PICKUP_ITEM2);
	savegame.num_puzzle1 = Inv_RequestItem(PUZZLE_ITEM1);
	savegame.num_puzzle2 = Inv_RequestItem(PUZZLE_ITEM2);
	savegame.num_puzzle3 = Inv_RequestItem(PUZZLE_ITEM3);
	savegame.num_puzzle4 = Inv_RequestItem(PUZZLE_ITEM4);
	savegame.num_key1 = Inv_RequestItem(KEY_ITEM1);
	savegame.num_key2 = Inv_RequestItem(KEY_ITEM2);
	savegame.num_key3 = Inv_RequestItem(KEY_ITEM3);
	savegame.num_key4 = Inv_RequestItem(KEY_ITEM4);

	ResetSG();

/* Clear out the savegame buffer */
	for ( i=0;i<MAX_SAVEGAME_BUFFER;i++ )
		savegame.buffer[i] = 0;

/* Save out all the other bits and bobs that are needed per level */
	WriteSG(&flip_status, sizeof(int));

/* Flipped map flags */
	for (i=0; i<MAX_FLIPMAPS; i++)
	{
		tflag = (flipmap[i] >> 8);
		WriteSG(&tflag, sizeof(char));
	}

/* Save out CD flags */
	WriteSG(cd_flags, sizeof(char)*MAX_CD_TRACKS);

/* Camera/sink flags */
	for (i=0; i<number_cameras; i++)
		WriteSG(&camera.fixed[i].flags, sizeof(sint16));

/*
 * Go through all the items in the level and save out the salient features
 * (as flagged in objects.h)
 */
	for ( i=0,item=items ; i<level_items ; i++,item++ )
	{
		obj = &objects[item->object_number];

		if ((item->object_number == TONY && item->hit_points > 0 && bossdata.explode_count) || bossdata.dead)
			noresetexplode = 1;

		if (obj->save_position)
		{
			WriteSG(&item->pos, sizeof(PHD_3DPOS));
			WriteSG(&item->room_number, sizeof(sint16));
#ifdef FULL_SAVE
			WriteSG(&item->speed, sizeof(sint16));
			WriteSG(&item->fallspeed, sizeof(sint16));
#endif
		}

		if (obj->save_anim)
		{
			WriteSG(&item->current_anim_state, sizeof(sint16));
#ifdef FULL_SAVE
			WriteSG(&item->goal_anim_state, sizeof(sint16));
			WriteSG(&item->required_anim_state, sizeof(sint16));
#endif
			WriteSG(&item->anim_number, sizeof(sint16));
			WriteSG(&item->frame_number, sizeof(sint16));
		}

		if (obj->save_hitpoints)
			WriteSG(&item->hit_points, sizeof(sint16));

		if (obj->save_flags)
		{
			uint32 flags=0;

			flags = ((uint16)item->flags)
				 | (item->active << 16)
				 | (item->status << 17)
				 | (item->gravity_status << 19)
				 | (item->hit_status << 20)
				 | (item->collidable << 21)
				 | (item->looked_at << 22)
				 | (item->dynamic_light << 23)
				 | (item->clear_body << 24)
				 | (item->ai_bits << 25)
				 | (item->really_active << 30);

			if (obj->intelligent && item->data)
				flags |= SAVE_CREATURE;

			WriteSG(&flags, sizeof(uint32));

		/* Intelligent baddies can be carrying stuff which they drop on death */

			if (obj->intelligent)
				WriteSG(&item->carried_item, sizeof(sint16));

		/* Write out timer + baddie AI stuff for PC */
#ifdef FULL_SAVE
			WriteSG(&item->timer, sizeof(sint16));

			if (flags & SAVE_CREATURE)
				WriteSG((CREATURE_INFO *)item->data, SAVE_CREATURE_SIZE);
#if 0
			{
				creature = (CREATURE_INFO *)item->data;
				WriteSG(&creature->joint_rotation[0], sizeof(sint16));
				WriteSG(&creature->joint_rotation[1], sizeof(sint16));
				WriteSG(&creature->joint_rotation[2], sizeof(sint16));
				WriteSG(&creature->joint_rotation[3], sizeof(sint16));
				WriteSG(&creature->maximum_turn, sizeof(sint16));
				WriteSG(&creature->flags, sizeof(sint16));
				WriteSG(&creature->mood, sizeof(MOOD_TYPE));
			}
#endif
#endif
		/* -------- save new item flags */

			WriteSG(&item->item_flags[0], sizeof(sint16)*4);
		}

	/* -------- save vehicle data */
#ifdef FULL_SAVE
		if (item->object_number == BIGGUN)
			WriteSG(item->data, sizeof(BIGGUNINFO));

		if (item->object_number == BOAT)
			WriteSG(item->data, sizeof(BOAT_INFO));

		if (item->object_number == KAYAK)
			WriteSG(item->data, sizeof(KAYAKINFO));

		if (item->object_number == MINECART)
			WriteSG(item->data, sizeof(CARTINFO));

		if (item->object_number == QUADBIKE)
			WriteSG(item->data, sizeof(QUADINFO));

		if (item->object_number == UPV)
			WriteSG(item->data, sizeof(SUBINFO));

	/* -------- save extra shit */

		if (item->object_number == AREA51_LASER)
			WriteSG(&item->shadeB, sizeof(sint16));
#endif
	}

	texplode = bossdata.explode_count;
	if (noresetexplode == 0)
		bossdata.explode_count = 0;
	WriteSG(&bossdata, sizeof(BOSS_STRUCT));
	bossdata.explode_count = texplode;

	for (i=0; i<15; i++)
		lara.mesh_ptrs[i] = (sint16 *)((long)lara.mesh_ptrs[i] - (long)mesh_base);
	lara.left_arm.frame_base = (sint16 *)((long)lara.left_arm.frame_base - (long)frames);
	lara.right_arm.frame_base = (sint16 *)((long)lara.right_arm.frame_base - (long)frames);

	WriteSG(&lara, sizeof(LARA_INFO));

	for (i=0; i<15; i++)
		lara.mesh_ptrs[i] = (sint16 *)((long)lara.mesh_ptrs[i] + (long)mesh_base);
	lara.left_arm.frame_base = (sint16 *)((long)lara.left_arm.frame_base + (long)frames);
	lara.right_arm.frame_base = (sint16 *)((long)lara.right_arm.frame_base + (long)frames);

#ifdef FULL_SAVE
	/* Is Lara using a new weapon (M16, harpoon, etc)? (won't be on PSX, as have to pick up crystal) */
	if (lara.weapon_item != NO_ITEM)
	{
		item = &items[lara.weapon_item];
		WriteSG(&item->object_number, sizeof(sint16));
		WriteSG(&item->anim_number, sizeof(sint16));
		WriteSG(&item->frame_number, sizeof(sint16));
		WriteSG(&item->current_anim_state, sizeof(sint16));
		WriteSG(&item->goal_anim_state, sizeof(sint16));
	}
#endif

	WriteSG(&flipeffect, sizeof(int));
	WriteSG(&fliptimer, sizeof(int));

	WriteSG(&CurrentAtmosphere, sizeof(char));
	WriteSG(&compy_scared_timer, sizeof(int));
	WriteSG(&compys_attack_lara, sizeof(int));
	WriteSG(&CarcassItem, sizeof(sint16));
	WriteSG(&TribeBossShieldOn, sizeof(char));
	WriteSG(&TribeBossItem, sizeof(sint32));
	WriteSG(&lizard_man_active, sizeof(char));


#ifdef FULL_SAVE
	/* First count up number of flares, then output their info */
	number_flares = 0;
	for (i=level_items, item=&items[level_items]; i<NUMBER_ITEMS; i++, item++)
		if (item->active && item->object_number == FLARE_ITEM)
			number_flares++;

		WriteSG(&number_flares, sizeof(int));
	for (i=level_items, item=&items[level_items]; i<NUMBER_ITEMS; i++, item++)
		if (item->active && item->object_number == FLARE_ITEM)
		{
			WriteSG(&item->pos, sizeof(PHD_3DPOS));
			WriteSG(&item->room_number, sizeof(sint16));
			WriteSG(&item->speed, sizeof(sint16));
			WriteSG(&item->fallspeed, sizeof(sint16));
			flare_age = (int)item->data;
			WriteSG(&flare_age, sizeof(int));
		}
#endif
#if defined(PSX_VERSION) && defined(GAMEDEBUG)
	printf("SGcount: %d\n", SGcount);
#endif
}


void ExtractSaveGameInfo(void)
{
	sint16 i;
	OBJECT_INFO *obj;
	ITEM_INFO *item;
	FLOOR_INFO *floor;
	sint16 room_number;
	char tflag;
#ifdef FULL_SAVE
	sint16 item_number;
	CREATURE_INFO *creature;
	int number_flares, flare_age;
#endif

#if defined(PSX_VERSION) && defined(GAMEDEBUG)
	printf("ExtractSaveGameInfo()\n");
#endif

#ifdef PSX_VERSION
	InitialiseLaraInventory(MAXIMUM_LEVELS-2);	// ## was 0 but now save Gym
#else
	InitialiseLaraInventory(CurrentLevel);
#endif

#if defined(PSX_VERSION) && defined(GAMEDEBUG)
	printf("Adding Keys/Puzzles\n");
#endif

	/* Initialise Local Level items such as Keys/Puzzle Items */
	for (i=savegame.num_pickup1; i>0; i--)
		Inv_AddItem( PICKUP_ITEM1 );

	for (i=savegame.num_pickup2; i>0; i--)
		Inv_AddItem( PICKUP_ITEM2 );

	for (i=savegame.num_puzzle1; i>0; i--)
		Inv_AddItem( PUZZLE_ITEM1 );

	for (i=savegame.num_puzzle2; i>0; i--)
		Inv_AddItem( PUZZLE_ITEM2 );

	for (i=savegame.num_puzzle3; i>0; i--)
		Inv_AddItem( PUZZLE_ITEM3 );

	for (i=savegame.num_puzzle4; i>0; i--)
		Inv_AddItem( PUZZLE_ITEM4 );

	for (i=savegame.num_key1; i>0; i--)
		Inv_AddItem( KEY_ITEM1 );

	for (i=savegame.num_key2; i>0; i--)
		Inv_AddItem( KEY_ITEM2 );

	for (i=savegame.num_key3; i>0; i--)
		Inv_AddItem( KEY_ITEM3 );

	for (i=savegame.num_key4; i>0; i--)
		Inv_AddItem( KEY_ITEM4 );

#if defined(PSX_VERSION) && defined(GAMEDEBUG)
	printf("Reading savegame buffer\n");
#endif
	/* Now load other stuff from savegame buffer */
	ResetSG();

	/* Flip map if need be */
	ReadSG(&i, sizeof(int));
	if (i)
		FlipMap();

/* Load flipmap flags */
	for (i=0; i<MAX_FLIPMAPS; i++)
	{
		ReadSG(&tflag, sizeof(char));
		flipmap[i] = (sint16)tflag << 8;
	}

/* CD track flags */
	ReadSG(cd_flags, sizeof(char) * MAX_CD_TRACKS);

/* Load fixed camera/sink flags */
	for (i=0; i<number_cameras; i++)
		ReadSG(&camera.fixed[i].flags, sizeof(sint16));

/* Go through all the items in the level and load in the salient features */
	for ( i=0,item=items ; i<level_items ; i++,item++ )
	{
		obj = &objects[item->object_number];

	/* MOVABLE BLOCK needs a little extra work due to its effect on the ground height */
		if (obj->control == MovableBlock)
			AlterFloorHeight(item, 1024);

		if (obj->save_position)
		{
			ReadSG(&item->pos, sizeof(PHD_3DPOS));
			ReadSG(&room_number, sizeof(sint16));
#ifdef FULL_SAVE
			ReadSG(&item->speed, sizeof(sint16));
			ReadSG(&item->fallspeed, sizeof(sint16));
#endif
			if (item->room_number != room_number)
				ItemNewRoom(i, room_number);

			/* Setup floor height for item's shadow */
			if (obj->shadow_size)
			{
				floor = GetFloor(item->pos.x_pos, item->pos.y_pos, item->pos.z_pos, &room_number);
				item->floor = GetHeight(floor, item->pos.x_pos, item->pos.y_pos, item->pos.z_pos);
			}
		}

		if (obj->save_anim)
		{
			ReadSG(&item->current_anim_state, sizeof(sint16));
#ifdef FULL_SAVE
			ReadSG(&item->goal_anim_state, sizeof(sint16));
			ReadSG(&item->required_anim_state, sizeof(sint16));
#endif
			ReadSG(&item->anim_number, sizeof(sint16));
			ReadSG(&item->frame_number, sizeof(sint16));
		}

		if (obj->save_hitpoints)
			ReadSG(&item->hit_points, sizeof(sint16));

		if (obj->save_flags)
		{
			uint32 flags;

			ReadSG(&flags, sizeof(uint32));

			item->flags = flags & 0xffff;

			if (obj->intelligent)
				ReadSG(&item->carried_item, sizeof(sint16));

#ifdef FULL_SAVE
			ReadSG(&item->timer, sizeof(sint16));
#endif

		/* Setup item flags, including activating baddies etc */

			if (item->flags & KILLED_ITEM)
			{
				KillItem(i);
				item->status = DEACTIVATED;
			}
			else
			{
				if ((flags & 0x10000) && !item->active)
					AddActiveItem(i);

				item->active = (flags >> 16) & 1;
				item->status = (flags >> 17) & 3;
				item->gravity_status = (flags >> 19) & 1;
				item->hit_status = (flags >> 20) & 1;
				item->collidable = (flags >> 21) & 1;
				item->looked_at = (flags >> 22) & 1;
				item->dynamic_light = (flags >> 23) & 1;
				item->clear_body = (flags >> 24) & 1;
				item->ai_bits = (flags >> 25) & 31;
				item->really_active = (flags >> 30) & 1;
			}

		/* Load in baddies intelligence stuff */
#ifdef FULL_SAVE
			if (flags & SAVE_CREATURE)
			{
				EnableBaddieAI(i, 1);
				creature = (CREATURE_INFO *)item->data;
				if (creature)
					ReadSG(creature, SAVE_CREATURE_SIZE);
#if 0
				{
	//				ReadSG(&creature->head_rotation, sizeof(sint16));
	//				ReadSG(&creature->neck_rotation, sizeof(sint16));
					ReadSG(&creature->joint_rotation[0], sizeof(sint16));
					ReadSG(&creature->joint_rotation[1], sizeof(sint16));
					ReadSG(&creature->joint_rotation[2], sizeof(sint16));
					ReadSG(&creature->joint_rotation[3], sizeof(sint16));
					ReadSG(&creature->maximum_turn, sizeof(sint16));
					ReadSG(&creature->flags, sizeof(sint16));
					ReadSG(&creature->mood, sizeof(MOOD_TYPE));
				}
#endif
				else
//##					SGpoint += sizeof(sint16)*4 + sizeof(MOOD_TYPE);
					SGpoint += SAVE_CREATURE_SIZE;
			}
			else if (obj->intelligent)
			{
				/* If not saved, but intelligent, means that currently no AI allocated */
				item->data = NULL;

				if (item->clear_body && item->hit_points<=0 && !(item->flags & KILLED_ITEM)) // uncleared body for the body bag
				{
					item->next_active = body_bag;
					body_bag = i;
				}
			}
#endif

//##			item->flags &= 0xff00;

			/* PUZZLE HOLES that have deactivated have changed to PUZZLE_DONE */
			if (obj->collision == PuzzleHoleCollision && (item->status==DEACTIVATED || item->status==ACTIVE))
				item->object_number += PUZZLE_DONE1 - PUZZLE_HOLE1;

			/* PICKUPs can be DEACTIVATED if used to trigger a pickup trigger, so need to be removed from draw list */
			if (obj->collision == PickUpCollision && item->status == DEACTIVATED)
				RemoveDrawnItem(i);

			/* SMASH_WINDOW and SMASH_OBJECT1-3 will be in broken state if ONESHOT bit is set */
			if ((item->object_number == SMASH_WINDOW ||
				item->object_number == SMASH_OBJECT1 ||
				item->object_number == SMASH_OBJECT2 ||
				item->object_number == SMASH_OBJECT3) &&
				(item->flags & ONESHOT))
				item->mesh_bits = 0x00100;

			/* MINE is detonated if ONESHOT set */
//			if (item->object_number == MINES && (item->flags & ONESHOT))
//				item->mesh_bits = 1;

		/* -------- load new item flags */

			ReadSG(&item->item_flags[0], sizeof(sint16)*4);
		}

		/* Setup MOVABLE BLOCK in new position */
		if (obj->control == MovableBlock && item->status == NOT_ACTIVE)
			AlterFloorHeight(item, -1024);
		else if (obj->control == MovableBlock && item->status != NOT_ACTIVE)
			SetupCleanerFromSavegame(item,1);

		if (item->object_number == ELECTRIC_CLEANER)
			SetupCleanerFromSavegame(item,0);

		if ((item->object_number == TROPICAL_FISH || item->object_number == PIRAHNAS) && item->hit_points != -1)
			SetupShoal(item->hit_points);

	/* -------- load vehicle data */
#ifdef FULL_SAVE
		if (item->object_number == BIGGUN)
			ReadSG(item->data, sizeof(BIGGUNINFO));

		if (item->object_number == BOAT)
			ReadSG(item->data, sizeof(BOAT_INFO));

		if (item->object_number == KAYAK)
			ReadSG(item->data, sizeof(KAYAKINFO));

		if (item->object_number == MINECART)
			ReadSG(item->data, sizeof(CARTINFO));

		if (item->object_number == QUADBIKE)
			ReadSG(item->data, sizeof(QUADINFO));

		if (item->object_number == UPV)
			ReadSG(item->data, sizeof(SUBINFO));

	/* -------- load extra shit */

		if (item->object_number == AREA51_LASER)
			ReadSG(&item->shadeB, sizeof(sint16));

	/* -------- sort out Monkey cunt */

		if ((item->object_number == MONKEY)
		&&  (item->carried_item != NO_ITEM))
		{
			RemoveDrawnItem(item->carried_item);
			items[item->carried_item].room_number = NO_ROOM;
			items[item->carried_item].carried_item = NO_ITEM;
		}

#endif
	}

	ReadSG(&bossdata, sizeof(BOSS_STRUCT));
	ReadSG(&lara, sizeof(LARA_INFO));

	lara.creature = NULL;
	lara.spaz_effect = NULL;
	for (i=0; i<15; i++)
		lara.mesh_ptrs[i] = (sint16 *)((long)lara.mesh_ptrs[i] + (long)mesh_base);
	lara.target = NULL;
	lara.left_arm.frame_base = (sint16 *)((long)lara.left_arm.frame_base + (long)frames);
	lara.right_arm.frame_base = (sint16 *)((long)lara.right_arm.frame_base + (long)frames);

/* -------- do a bit more mesh swapping ? */

	if (lara.skidoo != NO_ITEM)
	{
		if (items[lara.skidoo].object_number == MINECART)
		{
			CARTINFO *vi = (CARTINFO *)(items[lara.skidoo].data);

			if (vi->Flags & CF_MESH)
				meshes[objects[VEHICLE_ANIM].mesh_index + HAND_R] = meshes[objects[LARA_SKIN].mesh_index + HAND_R];
		}

		else if (items[lara.skidoo].object_number == KAYAK)
		{
			KAYAKINFO *vi = (KAYAKINFO *)(items[lara.skidoo].data);

			if (vi->Flags & 0x80)
				meshes[objects[VEHICLE_ANIM].mesh_index + HAND_R] = meshes[objects[LARA_SKIN].mesh_index + HAND_R];
		}
	}


#ifdef FULL_SAVE
	/* Is Lara using a new weapon (M16, harpoon, etc)? (won't be on PSX, as have to pick up crystal) */
	if (lara.weapon_item != NO_ITEM)
	{
		lara.weapon_item = CreateItem();
		item = &items[lara.weapon_item];
		ReadSG(&item->object_number, sizeof(sint16));
		ReadSG(&item->anim_number, sizeof(sint16));
		ReadSG(&item->frame_number, sizeof(sint16));
		ReadSG(&item->current_anim_state, sizeof(sint16));
		ReadSG(&item->goal_anim_state, sizeof(sint16));
		item->status = ACTIVE;
		item->room_number = NO_ROOM;
	}
#endif

	if (lara.burn) // user saved whilst on fire!
	{
		lara.burn = 0;
		LaraBurn();
	}

	ReadSG(&flipeffect, sizeof(int));
	ReadSG(&fliptimer, sizeof(int));

	ReadSG(&CurrentAtmosphere, sizeof(char));
	ReadSG(&compy_scared_timer, sizeof(int));
	ReadSG(&compys_attack_lara, sizeof(int));
	ReadSG(&CarcassItem, sizeof(sint16));
	ReadSG(&TribeBossShieldOn, sizeof(char));
	ReadSG(&TribeBossItem, sizeof(sint32));
	ReadSG(&lizard_man_active, sizeof(char));


#ifdef FULL_SAVE
	/* First count up number of flares, then output their info */
	ReadSG(&number_flares, sizeof(int));
	//printf("Number of flares=%d", number_flares);
	for (i=0; i<number_flares; i++)
	{
		item_number = CreateItem();
		item = &items[item_number];
		item->object_number = FLARE_ITEM;
		ReadSG(&item->pos, sizeof(PHD_3DPOS));
		ReadSG(&item->room_number, sizeof(sint16));
		ReadSG(&item->speed, sizeof(sint16));
		ReadSG(&item->fallspeed, sizeof(sint16));
		InitialiseItem(item_number);
		AddActiveItem(item_number);
		ReadSG(&flare_age, sizeof(int));
		item->data = (void *)flare_age;
	}
#endif
}


void ResetSG()
{
	SGcount = 0;
	SGpoint = savegame.buffer;
}


void WriteSG(void *pointer, int size)
{
	char *data;

	SGcount += size;
	if (SGcount >= MAX_SAVEGAME_BUFFER)
	{
#if defined(GAMEDEBUG) && defined(PSX_VERSION)
		printf("SGcount=%d (%d MAX)\n", SGcount, MAX_SAVEGAME_BUFFER);
#endif
		S_ExitSystem("FATAL: Savegame is too big to fit in buffer");
	}

	data = pointer;

	for ( ; size>0; size--)
		*(SGpoint++) = *(data++);
}


void ReadSG(void *pointer, int size)
{
	char *data;

	SGcount += size;

	data = pointer;

	for ( ; size>0; size--)
		*(data++) = *(SGpoint++);
}

#ifndef PSX_VERSION
int ConfirmSave(int fade)
{
	int	nframes;
	int status=1;
	TEXTSTRING *text[3]={NULL};

	T_InitPrint();
	if (fade)
	{
		S_FadeInInventory(1);
	}
	text[0] = T_Print(0,0,0,GF_PCStrings[PCSTR_SAVEMESS8]);
	T_CentreH(text[0], 1);
	T_CentreV(text[0], 1);

	text[1] = T_Print(-26,20,0,GF_PCStrings[PCSTR_SAVEMESS3]);
	T_CentreH(text[1], 1);
	T_CentreV(text[1], 1);
	T_AddOutline(text[1], 1, C_ORANGE, req_sel_gour2, 0);

	text[2] = T_Print(30,20,0,GF_PCStrings[PCSTR_SAVEMESS4]);
	T_CentreH(text[2], 1);
	T_CentreV(text[2], 1);

	while (input & IN_SELECT)
		S_UpdateInput();

	do
	{
		S_UpdateInput();
#ifdef PC_VERSION
		S_InitialisePolyList(SIPL_DONT_CLEAR_SCREEN);	// bgnd clears it
#else
		S_InitialisePolyList();
#endif
		S_CopyBufferToScreen();
		if (input&IN_LEFT && status==0)
		{
			T_AddOutline(text[1], 1, C_ORANGE, req_sel_gour2, 0);
			T_RemoveOutline(text[2]);
			status=1;
		}
		if (input&IN_RIGHT && status==1)
		{
			T_RemoveOutline(text[1]);
			T_AddOutline(text[2], 1, C_ORANGE, req_sel_gour2, 0);
			status=0;
		}

		T_DrawText();
		S_OutputPolyList();
		nframes = S_DumpScreen();
	}
	while(!(input&IN_SELECT));

	input = 0;

	while (input & IN_SELECT)
		S_UpdateInput();

	T_RemovePrint(text[0]); text[0] = NULL;
	T_RemovePrint(text[1]); text[1] = NULL;
	T_RemovePrint(text[2]); text[2] = NULL;

	return(status);
}

void CompleteSave(int fade)
{
	if (fade)
	{
		S_FadeOutInventory(1);
	}
}
#endif
/********************************************************************
 *			create the save game checksum
 *******************************************************************/
#ifdef PSX_VERSION
void	CreateCheckSum( SAVEGAME_INFO *sg )
{
	int		i;
	signed char	*ptr,csum;

	if ( sg==NULL )
		return;
	csum = 0;
	sg->checksum = 0;
	ptr = (signed char *)sg;
	for ( i=0; i<sizeof(SAVEGAME_INFO); i++,ptr++ )
		csum += *ptr;
	sg->checksum = -csum;
}
#else
void	CreateCheckSum(void)
{
	int		i;
	signed char	*ptr,csum;

	csum = 0;
	savegame.checksum = 0;
	ptr = (signed char *)&savegame;
	for ( i=0; i<sizeof(SAVEGAME_INFO); i++,ptr++ )
		csum += *ptr;
	savegame.checksum = -csum;
}
#endif

/********************************************************************
 *			Check If the Savegame is Valid
 *******************************************************************/
int		CheckSumValid( SAVEGAME_INFO *buffer )
{
	int		i;
	signed char	*ptr,csum;

	csum = 0;
	ptr = (signed char *)buffer;
	for ( i=0; i<sizeof(SAVEGAME_INFO); i++,ptr++ )
		csum += *ptr;
	if ( !csum )
		return(1);
	return(0);
}
