/****************************************************************************
*
* SNOW.C
*
*    VERSION : 00.00
*    CREATED : 2/07/98
*   MODIFIED : 2/07/98
*	   TABS : 5
*
* DESCRIPTION
*
*****************************************************************************/


/*---------------------------------------------------------------------------
 * Import Headers
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
#include "..\spec_psx\typedefs.h"
#include "..\spec_psx\gpu.h"
#include "..\spec_psx\psxinput.h"
#include <libgte.h>
#include <libgpu.h>
#include <gtemac.h>
#include <inline_c.h>
#else
#include "../specific/standard.h"
#include "../specific/stypes.h"
#endif

#ifdef PSX_VERSION
#include "..\spec_psx\maths.h"
#endif

#include "lara.h"
#include "snow.h"
#include "anim.h"
#include "objects.h"
#include "control.h"
#include "input.h"

/*---------------------------------------------------------------------------
 * Constants
\*--------------------------------------------------------------------------*/

#define SNOW_COLLISION

#define X 0
#define Y 1
#define Z	2
#define XYZ 3

#define CENTREX 256
#define CENTREY 120

/*---------------------------------------------------------------------------
 * Externals
\*--------------------------------------------------------------------------*/


#ifdef PSX_VERSION
	extern VECTOR3D CamPos, CamTgt;
#endif

extern WORD rcossin_tbl[];
extern	long	SmokeWindX, SmokeWindZ;
extern	short IsRoomOutsideNo;

/*---------------------------------------------------------------------------
 * Globals
\*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
 * Locals
\*--------------------------------------------------------------------------*/

static SNOWFLAKE	snowflakes[MAX_SNOWFLAKES];

/*---------------------------------------------------------------------------
 * Macros
\*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
 * Local Functions
\*--------------------------------------------------------------------------*/

/****************************************************************************
* DoSnow
*
* SYNOPSIS
*	void DoSnow()
*
* PURPOSE
*	Displays a load of lovely snowflakes
*
* INPUTS
*	None
*
* RETURNS
*	Nothing
*/

#ifdef PSX_VERSION

void DoSnow()
{
	long	x, y, z, radius, lp, angle, u1, u2, v1, v2, size, out, third, triggered;
	SNOWFLAKE	*snowptr;
	PSXSPRITESTRUCT *pSpriteInfo;
	POLY_FT3 	*ft3;

	short	*scrxy;	// X,Y
	long		*scrz;	// Z
	short	*TempMesh;

	if (input & IN_DASH)
		return;

	scrxy = (short *) 0x1f800000;	// Set pointer to scratch pad.
	scrz = (long *) 0x1f800100;	// Set pointer to scratch pad.
	TempMesh = (short *) 0x1f800200;	// Set pointer to scratch pad.

	triggered = 0;

// Update snowflakes and trigger any if need be (i.e. Xpos = 0).

	snowptr = &snowflakes[0];

	for ( lp=0 ; lp<MAX_SNOWFLAKES ; lp++ )
	{
		if (snowptr->x == 0 && triggered < 8)	// Trigger a new one.
		{
			triggered++;
			y = -1024 - (GetRandomDraw()&2047);

			radius = (GetRandomDraw()&4095);
			angle = GetRandomDraw()&8190;

			x = (((rcossin_tbl[angle])*radius)>>12);
			z = (((rcossin_tbl[angle+1])*radius)>>12);

			snowptr->x = lara_item->pos.x_pos+x;
			snowptr->y = lara_item->pos.y_pos+y;
			snowptr->z = lara_item->pos.z_pos+z;
			if (IsRoomOutside(snowptr->x, snowptr->y, snowptr->z) < 0)
			{
				snowptr->x = 0;
				snowptr++;
				continue;
			}
			snowptr->stopped = 0;
			snowptr->xv = (GetRandomDraw()&7) - 4;
			snowptr->yv = ((GetRandomDraw()%24) + 8)<<3;
			snowptr->zv = (GetRandomDraw()&7) - 4;
			snowptr->life = 32+(64-(snowptr->yv<<1));
		}

		x = snowptr->x;
		y = snowptr->y;
		z = snowptr->z;

		if (!snowptr->stopped)	// Hit a surface ?
		{
			snowptr->x+=snowptr->xv;
			snowptr->y+=(snowptr->yv&(~7))>>2;
			snowptr->z+=snowptr->zv;

			out = IsRoomOutside(snowptr->x, snowptr->y, snowptr->z);
			if (out == -3)	// Inside?
			{
				snowptr->x = 0;
				snowptr++;
				continue;
			}
			else if (out == -2 || room[IsRoomOutsideNo].flags & UNDERWATER)	// Underwater room or hit wall/floor/ceiling.
			{
				snowptr->stopped = 1;
				snowptr->x = x;
				snowptr->y = y;
				snowptr->z = z;
				if (snowptr->life > 16)
					snowptr->life = 16;
				if (snowptr->yv > 16)
					snowptr->yv -= 16;
			}
		}

		if (!snowptr->life ||
			abs(CamPos.x - snowptr->x) > 6000 ||
			abs(CamPos.z - snowptr->z) > 6000 )
		{
			if (!snowptr->life)
			{
				snowptr->x = 0;	// Turn it off.
				snowptr++;
				continue;
			}
			else if (snowptr->life > 16)
				snowptr->life = 16;
		}

		if (snowptr->xv < (SmokeWindX<<1))
			snowptr->xv++;
		else if (snowptr->xv > (SmokeWindX<<1))
			snowptr->xv--;

		if (snowptr->zv < (SmokeWindZ<<1))
			snowptr->zv++;
		else if (snowptr->zv > (SmokeWindZ<<1))
			snowptr->zv--;

		snowptr->life-=2;
		if ((snowptr->yv&7) < 7)
			snowptr->yv++;
		snowptr++;
	}

	pSpriteInfo=psxspriteinfo+objects[EXPLOSION1].mesh_index+17;
	u1 = pSpriteInfo->u1;
	u2 = pSpriteInfo->u2;
	v1 = pSpriteInfo->v1;
	v2 = pSpriteInfo->v2;

	mPushMatrix();
	mTranslateAbsXYZ(lara_item->pos.x_pos,lara_item->pos.y_pos,lara_item->pos.z_pos);

	ft3=(POLY_FT3*)db.polyptr;

	snowptr = &snowflakes[0];

	for ( lp=0 ; lp<MAX_SNOWFLAKES ; lp++ )
	{
		if ((char *)ft3 >= db.polybuf_limit)
		{
			db.polyptr = (char *)ft3;
			mPopMatrix();
			return;
		}

		if (snowptr->x == 0)	// Not triggered?
		{
			snowptr++;
			continue;
		}

		TempMesh[X] = snowptr->x-lara_item->pos.x_pos;
		TempMesh[Y] = snowptr->y-lara_item->pos.y_pos;
		TempMesh[Z] = snowptr->z-lara_item->pos.z_pos;

  		gte_ldv0(TempMesh);
		gte_rtps();
		gte_stsxy(scrxy);
		gte_stsz(scrz);

		if ((*scrz) > 16384 || (*scrz) < 32 ||
			scrxy[X] < -32 ||
			scrxy[X] > 512+32 ||
			scrxy[Y] < -64 ||
			scrxy[Y] > 256+64)
		{
			if ((*scrz) < 32)
			{
				if (snowptr->life > 16)
			 		snowptr->life = 16;
			}
			snowptr++;
			continue;
		}

		size = (((snowptr->yv>>3))*phd_persp) / ((*scrz)>>2);
		if (size < 4)
			size = 4;
		else	if (size > 16)
			size = 16;

		third = (size * 0x5556)>>16;
		setlen(ft3, 7);
		setcode(ft3, 0x26);	// Transparent ft3.
		setXY3(ft3,scrxy[X]+third,scrxy[Y]-(third<<1),scrxy[X]+third,scrxy[Y]+third,scrxy[X]-(third<<1),scrxy[Y]+third);
		setUV3(ft3,u2,v1+1,u2,v2,u1+1,v2);

		if ((snowptr->yv&7) < 7)	// Fade in.
		{
			long	fadeval = (snowptr->yv&7)<<3;
			setRGB0(ft3,fadeval,fadeval,fadeval);
		}
		else if (snowptr->life > 18)	// Normal.
			setRGB0(ft3,72,72,72);
		else
			setRGB0(ft3,snowptr->life<<2,snowptr->life<<2,snowptr->life<<2);	// Fade out.
		ft3->clut = pSpriteInfo->clut;
		ft3->tpage = (pSpriteInfo->tpage & (~96)) | (1<<5);	// Set Colour addition.
		MyaddPrim( db.ot,((*scrz)>>3)+48, ft3 );
		ft3++;

		snowptr++;
	}
	db.polyptr = (uchar *) ft3;

	mPopMatrix();
}

#else
/*
void DoSnow()
{
}
*/
#endif

