/*********************************************************************************************/
/*                                                                                           */
/* Camera Control Code                                                                       */
/*                                                                                           */
/*********************************************************************************************/

/*---------------------------------------------------------------------------
 *	Import Headers
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
#include "..\spec_psx\typedefs.h"
#include "..\spec_psx\specific.h"
#include "..\spec_psx\maths.h"
#else
#include "..\specific\standard.h"
#include "..\specific\global.h"
#endif

#include "lara.h"
#include "laraanim.h"
#include "camera.h"
#include "control.h"
#include "sound.h"
#include "savegame.h"
#include "cinema.h"
#include "effect2.h"

#include "..\spec_psx\psxinput.h"
#include "..\game\input.h"

/*---------------------------------------------------------------------------
 *	Constants
\*--------------------------------------------------------------------------*/

long	CameraCollisionBounds(GAME_VECTOR *ideal, long push);
long mgLOS(GAME_VECTOR *start, GAME_VECTOR *target);

#define COM_SQUARE SQUARE(WALL_L/2)
#define MIN_SQUARE SQUARE(WALL_L/3)
#define MIN_CAM_ANGLE (ONE_DEGREE/2)

#define GROUND_SHIFT (STEP_L)

#define MAX_ELEVATION (ONE_DEGREE*85)

#define NORMAL_ELEVATION 	-(ONE_DEGREE*15)
/* Look around limits for automatic looking at targets */
#define MAX_LOOKX_ROTATION	-(75*ONE_DEGREE)
#define MIN_LOOKX_ROTATION	(55*ONE_DEGREE)
#define MAX_LOOKY_ROTATION	(80*ONE_DEGREE)
#define MAX_HEAD_ROTATION 	(50*ONE_DEGREE)
//#define MAX_HEAD_TILT	  	(25*ONE_DEGREE)
//#define MIN_HEAD_TILT 	  	(-45*ONE_DEGREE)
#define MAX_HEAD_TILT	  	(85*ONE_DEGREE)
#define MIN_HEAD_TILT 	  	(-85*ONE_DEGREE)
#define HEAD_TURN 		  	(4*ONE_DEGREE) //double what's needed because lara.c will reset it 2 degrees

/* Divider speeds for cameras to move to target i.e. pos += distance / speed */
#define CHASE_SPEED 10
#define COMBAT_SPEED 8
#define COMBAT_DISTANCE (WALL_L*3/2)
#define LOOK_SPEED   4

#define	MAX_CAMERA_MOVE	384
#define	MAX_LOOKCAMERA_MOVE	224

typedef struct
{
	sint16 current_anim_state;
	sint16 goal_anim_state;
	sint16 room_number;
	PHD_3DPOS pos;
} OLD_CAMERA;


/*---------------------------------------------------------------------------
 *	Globals
\*--------------------------------------------------------------------------*/

sint32	oldtx,oldty,oldtz;
sint32	TooFarCount = 0;

CAMERA_INFO camera;

PHD_VECTOR	last_rotations;	// For look camera.
PHD_VECTOR	static_lookcamp;	// For look camera.
PHD_VECTOR	static_lookcamt;	// For look camera.
OLD_CAMERA	old_cam;			// For normal camera to stop juddering.

char UseForcedFixedCamera;
GAME_VECTOR ForcedFixedCamera;

long NumFixedCameras;

/*********************************** FUNCTION CODE *******************************************/

void InitialiseCamera(void)
{
	/* Initialise position of camera */
	camera.target.x = lara_item->pos.x_pos;
	camera.target.y = camera.shift = lara_item->pos.y_pos-WALL_L;
	camera.target.z = lara_item->pos.z_pos;
	camera.target.room_number = lara_item->room_number;

	camera.pos.x = camera.target.x;
	camera.pos.y = camera.target.y;
	camera.pos.z = camera.target.z-100;
	camera.pos.room_number = camera.target.room_number;

	camera.target_distance = WALL_L*3/2;
	camera.item = NULL;
	camera.min_ypos = NO_MINY;

	camera.number_frames = 1;
	if (!lara.extra_anim)
		camera.type = CHASE_CAMERA;
	camera.speed = 1;
	camera.flags = 0;

	camera.bounce = 0;

	/* No fixed camera */
	camera.number = NO_CAMERA;
	camera.fixed_camera = 0;
	AlterFOV(GAME_FOV*ONE_DEGREE);

	UseForcedFixedCamera = 0;

	CalculateCamera();
}


void MoveCamera(GAME_VECTOR *ideal, int speed)
{
	FLOOR_INFO *floor;
	sint32 height, ceiling, shake;
	sint32 dx,dy,dz,ox,oy,oz,orn;

#define STRAIGHT_MOVE
#ifdef STRAIGHT_MOVE

	ox = camera.pos.x;
	oy = camera.pos.y;
	oz = camera.pos.z;
	orn = camera.pos.room_number;

	if (	speed != 1 && (camera.type == CHASE_CAMERA || camera.type == COMBAT_CAMERA) &&
	   (	abs(camera.pos.x - ideal->x) > 3072 ||
	   	abs(camera.pos.y - ideal->y) > 3072 ||
		abs(camera.pos.z - ideal->z) > 3072))	// If camera too far away, snap.
	{
		TooFarCount++;
		if (TooFarCount > 8)
		{
			TooFarCount = 0;
			speed = 1;
		}
	}
	else
		TooFarCount = 0;

	/* Move along straight line to ideal position */

	if (speed != 1)
	{
		dx = dy = dz = 512;
		while (abs(dx) > MAX_CAMERA_MOVE || abs(dy) > 255 || abs(dz) > MAX_CAMERA_MOVE)
		{
			dx = (ideal->x - camera.pos.x) / speed;
			dy = (ideal->y - camera.pos.y) / speed;
			dz = (ideal->z - camera.pos.z) / speed;
			speed++;
		}

		camera.pos.x += dx;
		camera.pos.y += dy;
		camera.pos.z += dz;
		camera.pos.room_number = ideal->room_number;
	}
	else
	{
		camera.pos.x = ideal->x;
		camera.pos.y = ideal->y;
		camera.pos.z = ideal->z;
		camera.pos.room_number = ideal->room_number;
	}
#else
	/* Move along radius to ideal position */
	sint32 x, z, distance, c_distance;
	sint16 angle, c_angle;

	/* Move along straight line to ideal position */

	z = ideal->z - camera.target.z;
	x = ideal->x - camera.target.x;
	angle = phd_atan(z, x);
	distance = mSqrt(z*z + x*x);

	// TODO: + need current angle and rotation, so worth adding to camera structure
	z = camera.pos.z - camera.target.z;
	x = camera.pos.x - camera.target.x;
	c_angle = phd_atan(z, x);
	c_distance = mSqrt(z*z + x*x);

	/* Linearly change both angle and distance to ideal values */
	angle -= c_angle;
	c_angle += angle / speed;
	c_distance += (distance - c_distance) / speed;
	camera.pos.z = camera.target.z + (c_distance * phd_cos(c_angle) >> W2V_SHIFT);
	camera.pos.x = camera.target.x + (c_distance * phd_sin(c_angle) >> W2V_SHIFT);
	camera.pos.y += (ideal->y - camera.pos.y) / speed;
	camera.pos.room_number = ideal->room_number;
#endif

	floor = GetFloor(camera.pos.x, camera.pos.y, camera.pos.z, &camera.pos.room_number);
	height = GetHeight(floor, camera.pos.x, camera.pos.y, camera.pos.z) - GROUND_SHIFT;

	/* This position is not valid so cut using LOS */
	if (camera.pos.y >= height && ideal->y >= height)
	{
		mgLOS(&camera.target, &camera.pos);
		floor = GetFloor(camera.pos.x, camera.pos.y, camera.pos.z, &camera.pos.room_number);
		height = GetHeight(floor, camera.pos.x, camera.pos.y, camera.pos.z) - GROUND_SHIFT;
	}

	/* Floor/ceiling check to avoid going underground */
	ceiling = GetCeiling(floor, camera.pos.x, camera.pos.y, camera.pos.z) + GROUND_SHIFT;
	if (height < ceiling)
		height = ceiling = (height + ceiling) >> 1;

	/* Add bounce effect */
	if (camera.bounce)
	{
		if (camera.bounce > 0)
		{
			camera.pos.y += camera.bounce;
			camera.target.y += camera.bounce;
			camera.bounce = 0;
		}
		else
		{
			/* Shake camera randomly */
			shake = (GetRandomControl()-0x4000) * camera.bounce / 0x7fff;
			camera.pos.x += shake;
			camera.target.y += shake;
			shake = (GetRandomControl()-0x4000) * camera.bounce / 0x7fff;
			camera.pos.y += shake;
			camera.target.y += shake;
			shake = (GetRandomControl()-0x4000) * camera.bounce / 0x7fff;
			camera.pos.z += shake;
			camera.target.z += shake;
			camera.bounce += 5;
		}
	}

	{
		sint16	room_number;
		long		wx,wy,wz;

		wx = camera.pos.x;
		wy = camera.pos.y;
		wz = camera.pos.z;
		room_number = camera.pos.room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		height=GetHeight(floor,wx,wy,wz);
		room_number = camera.pos.room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		ceiling=GetCeiling(floor,wx,wy,wz);
		if (wy < ceiling || wy > height)
		 	mgLOS(&camera.target,&camera.pos);
	}

	{
		sint16	room_number;
		long		wx,wy,wz;

		wx = camera.pos.x;
		wy = camera.pos.y;
		wz = camera.pos.z;
		room_number = camera.pos.room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		height=GetHeight(floor,wx,wy,wz);
		room_number = camera.pos.room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		ceiling=GetCeiling(floor,wx,wy,wz);
		if (wy < ceiling || wy > height)
			printf("?");
	}

	phd_LookAt(camera.pos.x, camera.pos.y, camera.pos.z, camera.target.x, camera.target.y, camera.target.z, 0);

	/* Either: shift target and viewpoint -> juddering	up and down + total loss of Lara from view
	           shift viewpoint only -> weird view angles */
//#ifdef SHIFT_BOTH
//	phd_LookAt(camera.pos.x, camera.pos.y + camera.shift, camera.pos.z,
//				  camera.target.x, camera.target.y + camera.shift, camera.target.z, 0);
//#else
//	phd_LookAt(camera.pos.x, camera.pos.y+camera.shift, camera.pos.z,
//			 camera.target.x, camera.target.y, camera.target.z, 0);
//#endif

	/* For stereo sound effects need to know camera facing */
#ifdef PC_VERSION
	if (camera.mike_at_lara)
	{
		camera.actual_angle=lara_item->pos.y_rot+lara.head_y_rot+lara.torso_y_rot;
		camera.mike_pos.x=lara_item->pos.x_pos;
		camera.mike_pos.y=lara_item->pos.y_pos;
		camera.mike_pos.z=lara_item->pos.z_pos;
	}
	else
#endif
	{
		camera.actual_angle = phd_atan(camera.target.z - camera.pos.z, camera.target.x - camera.pos.x);
		camera.mike_pos.x=camera.pos.x+((phd_persp*phd_sin(camera.actual_angle))>>W2V_SHIFT);
		camera.mike_pos.z=camera.pos.z+((phd_persp*phd_cos(camera.actual_angle))>>W2V_SHIFT);
		camera.mike_pos.y=camera.pos.y;
	}

	camera.old_type = camera.type;
}


void ClipCamera(sint32 *x, sint32 *y, sint32 *h, sint32 target_x, sint32 target_y, sint32 target_h,
					 sint32 left, sint32 top, sint32 right, sint32 bottom)
{
	/* Don't do either clip if target point is outside of bounds too */

	/* Clip to left first; failing that, may need a shift clamp */
	if ((right > left) ^ (target_x < left))
	{
		*y = target_y + (*y - target_y) * (left - target_x) / (*x - target_x);
		*h = target_h + (*h - target_h) * (left - target_x) / (*x - target_x);
		*x = left;
	}

	/* May need clip to top as well */
	if ((bottom > top && target_y > top && *y < top) ||
		 (bottom < top && target_y < top && *y > top))
	{
		*x = target_x + (*x - target_x) * (top - target_y) / (*y - target_y);
		*h = target_h + (*h - target_h) * (top - target_y) / (*y - target_y);
		*y = top;
	}
}


void ShiftCamera(sint32 *x, sint32 *y, sint32 *h, sint32 target_x, sint32 target_y, sint32 target_h,
					  sint32 left, sint32 top, sint32 right, sint32 bottom)
{
	/* Routine to move camera to ideal clipped position - assumes that position is clipping to left
		and that target position is lower than camera, so secondary clip to top.
		BEWARE: Negative square roots occur if target is further from clip edge than camera! */
	sint32 shift, TL_square, BL_square, TR_square;

	TL_square = SQUARE(target_x - left) + SQUARE(target_y - top);
	BL_square = SQUARE(target_x - left) + SQUARE(target_y - bottom);
	TR_square = SQUARE(target_x - right) + SQUARE(target_y - top);

	if (camera.target_square < TL_square)
	{
		/* Ideal position */
		*x = left;
		shift = camera.target_square - SQUARE(target_x - left);
		if (shift < 0)
			return;
		shift = phd_sqrt(shift);
		*y = target_y + ((top < bottom)? -shift : shift);
	}
	else if (TL_square > MIN_SQUARE)
	{
		/* Jammed to corner instead */
		*x = left;
		*y = top;
	}
	else if (camera.target_square < BL_square)
	{
		/* Shifted in opposite direction, but against primary clipping edge */
		*x = left;
		shift = camera.target_square - SQUARE(target_x - left);
		if (shift < 0)
			return;
		shift = phd_sqrt(shift);
		*y = target_y + ((top < bottom)? shift : -shift);
	}
	else if ((camera.target_square<<1) < TR_square)
	{
		/* Shifted along secondary clip edge; but further away so you can see in front of Lara */
		shift = (camera.target_square<<1) - SQUARE(target_y - top);
		if (shift < 0)
			return;
		shift = phd_sqrt(shift);
		*x = target_x + ((left < right)? shift : -shift);
		*y = top;
	}
	else if (BL_square > TR_square) // may want BL_square + a chunk > TR_square, to favour BL when close
	{
		/* If opposite corner on prime edge is further away, then secondary edge corner, use that */
		*x = left;
		*y = bottom;
	}
	else
	{
		/* Jammed to corner on secondary edge */
		*x = right;
		*y = top;
		}
}


FLOOR_INFO *GoodPosition(sint32 x, sint32 y, sint32 z, sint16 room_number)
{
	/* Return floor or NULL if not a good position */
	FLOOR_INFO *floor;

	floor = GetFloor(x, y, z, &room_number);
	if (y > GetHeight(floor, x, y, z) || y < GetCeiling(floor, x, y, z))
		return (NULL);

	return (floor);
}

void SmartShift(GAME_VECTOR *ideal,
	void (*shift)(sint32 *x, sint32 *y, sint32 *h, sint32 target_x, sint32 target_y, sint32 target_h,
					  sint32 left, sint32 top, sint32 right, sint32 bottom))
{
	int noclip, preferA;
	ROOM_INFO *r;
	sint16 item_box, camera_box;
	BOX_INFO *box;
	int x_floor, y_floor;
	sint32 left, right, top, bottom, test, edge;
	FLOOR_INFO *good_left, *good_right, *good_top, *good_bottom;
	GAME_VECTOR A, B;

	LOS(&camera.target, ideal);

	r = &room[camera.target.room_number];
	x_floor = (camera.target.z - r->z) >> WALL_SHIFT;
	y_floor = (camera.target.x - r->x) >> WALL_SHIFT;
	item_box = r->floor[x_floor + y_floor * r->x_size].box;

	/* Use item's box, unless camera is not in it */
	box = &boxes[item_box];

	left = (sint32)box->left << WALL_SHIFT;
	right = ((sint32)box->right << WALL_SHIFT) - 1;
	top = (sint32)box->top << WALL_SHIFT;
	bottom = ((sint32)box->bottom << WALL_SHIFT) - 1;

	/* Get camera box, and do bounding box checks to get improved camera position */
	r = &room[ideal->room_number];
	x_floor = (ideal->z - r->z) >> WALL_SHIFT;
	y_floor = (ideal->x - r->x) >> WALL_SHIFT;
	camera_box = r->floor[x_floor + y_floor * r->x_size].box;

	/* BUG: Can get dodgy LOS results - best to ignore them */
	if (camera_box != NO_BOX && (ideal->z < left || ideal->z > right || ideal->x < top || ideal->x > bottom))
	{
		box = &boxes[camera_box];
		left = (sint32)box->left << WALL_SHIFT;
		right = ((sint32)box->right << WALL_SHIFT) - 1;
		top = (sint32)box->top << WALL_SHIFT;
		bottom = ((sint32)box->bottom << WALL_SHIFT) - 1;
	}

	/* Check for possible box enlargements (or constrictions caused by low roof) around camera position */
	test = (ideal->z - WALL_L) | (WALL_L-1);
	good_left = GoodPosition(ideal->x, ideal->y, test, ideal->room_number);
	if (good_left)
	{
		camera_box = good_left->box;
		edge = (sint32)boxes[camera_box].left << WALL_SHIFT;
		if (camera_box != NO_ITEM && edge < left)
			left = edge;
	}
	else
		left = test;

	test = (ideal->z + WALL_L) & ~(WALL_L-1);
	good_right = GoodPosition(ideal->x, ideal->y, test, ideal->room_number);
	if (good_right)
	{
		camera_box = good_right->box;
		edge = ((sint32)boxes[camera_box].right << WALL_SHIFT) - 1;
		if (camera_box != NO_ITEM && edge > right)
			right = edge;
	}
	else
		right = test;

	test = (ideal->x - WALL_L) | (WALL_L-1);
	good_top = GoodPosition(test, ideal->y, ideal->z, ideal->room_number);
	if (good_top)
	{
		camera_box = good_top->box;
		edge = (sint32)boxes[camera_box].top << WALL_SHIFT;
		if (camera_box != NO_ITEM && edge < top)
			top = edge;
	}
	else
		top = test;

	test = (ideal->x + WALL_L) & ~(WALL_L-1);
	good_bottom = GoodPosition(test, ideal->y, ideal->z, ideal->room_number);
	if (good_bottom)
	{
		camera_box = good_bottom->box;
		edge = ((sint32)boxes[camera_box].bottom << WALL_SHIFT) - 1;
		if (camera_box != NO_ITEM && edge > bottom)
			bottom = edge;
	}
	else
		bottom = test;

	left += STEP_L+(STEP_L>>1);
	right -= STEP_L+(STEP_L>>1);
	top += STEP_L+(STEP_L>>1);
	bottom -=STEP_L+(STEP_L>>1);

	A.x = B.x = ideal->x;
	A.y = B.y = ideal->y;
	A.z = B.z = ideal->z;
	A.room_number = B.room_number = ideal->room_number;

	/* Shift camera position around according to clip against bounding box */
	noclip = preferA = 1;
	if (ABS(ideal->z - camera.target.z) > ABS(ideal->x - camera.target.x))
	{
		/* z axis is longest, check left/right clip first */
		if (ideal->z < left && !good_left)
		{
			noclip = 0;
			/* (A) Camera favours the side that the camera is currently on */
			preferA = (camera.pos.x < camera.target.x);
			(*shift)(&A.z, &A.x, &A.y, camera.target.z, camera.target.x, camera.target.y, left, top, right, bottom);
			(*shift)(&B.z, &B.x, &B.y, camera.target.z, camera.target.x, camera.target.y, left, bottom, right, top);
		}
		else if (ideal->z > right && !good_right)
		{
			noclip = 0;
			preferA = (camera.pos.x < camera.target.x);
			(*shift)(&A.z, &A.x, &A.y, camera.target.z, camera.target.x, camera.target.y, right, top, left, bottom);
			(*shift)(&B.z, &B.x, &B.y, camera.target.z, camera.target.x, camera.target.y, right, bottom, left, top);
		}

		/* If it didn't clip left/right, maybe it will top/bottom */
		if (noclip)
		{
			/* (B) Camera favours the side where the ideal position is on */
			if (ideal->x < top && !good_top)
			{
				noclip = 0;
				preferA = (ideal->z < camera.target.z);
				(*shift)(&A.x, &A.z, &A.y, camera.target.x, camera.target.z, camera.target.y, top, left, bottom, right);
				(*shift)(&B.x, &B.z, &B.y, camera.target.x, camera.target.z, camera.target.y, top, right, bottom, left);
			}
			else if (ideal->x > bottom && !good_bottom)
			{
				noclip = 0;
				preferA = (ideal->z < camera.target.z);
				(*shift)(&A.x, &A.z, &A.y, camera.target.x, camera.target.z, camera.target.y, bottom, left, top, right);
				(*shift)(&B.x, &B.z, &B.y, camera.target.x, camera.target.z, camera.target.y, bottom, right, top, left);
			}
		}
	}
	else
	{
		/* x axis is longest, check top/bottom clip first */
		if (ideal->x < top && !good_top)
		{
			noclip = 0;
			preferA = (camera.pos.z < camera.target.z);
			(*shift)(&A.x, &A.z, &A.y, camera.target.x, camera.target.z, camera.target.y, top, left, bottom, right);
			(*shift)(&B.x, &B.z, &B.y, camera.target.x, camera.target.z, camera.target.y, top, right, bottom, left);
		}
		else if (ideal->x > bottom && !good_bottom)
		{
			noclip = 0;
			preferA = (camera.pos.z < camera.target.z);
			(*shift)(&A.x, &A.z, &A.y, camera.target.x, camera.target.z, camera.target.y, bottom, left, top, right);
			(*shift)(&B.x, &B.z, &B.y, camera.target.x, camera.target.z, camera.target.y, bottom, right, top, left);
		}

		/* If it didn't clip top/bottom, maybe it will left/right */
		if (noclip)
		{
			if (ideal->z < left && !good_left)
			{
				noclip = 0;
				preferA = (ideal->x < camera.target.x);
				(*shift)(&A.z, &A.x, &A.y, camera.target.z, camera.target.x, camera.target.y, left, top, right, bottom);
				(*shift)(&B.z, &B.x, &B.y, camera.target.z, camera.target.x, camera.target.y, left, bottom, right, top);
			}
			else if (ideal->z > right && !good_right)
			{
				noclip = 0;
				preferA = (ideal->x < camera.target.x);
				(*shift)(&A.z, &A.x, &A.y, camera.target.z, camera.target.x, camera.target.y, right, top, left, bottom);
				(*shift)(&B.z, &B.x, &B.y, camera.target.z, camera.target.x, camera.target.y, right, bottom, left, top);
			}
		}
	}

	/* May not have needed adjustment */
	if (noclip)
		return;

	/* Check LOS; if bad, then use second choice instead */
	if (preferA && !LOS(&camera.target, &A))
		preferA = 0;
	else	if (!preferA && !LOS(&camera.target, &B))
		preferA = 1;

	if (preferA)
	{
		ideal->x = A.x;
		ideal->y = A.y;
		ideal->z = A.z;
	}
	else
	{
		ideal->x = B.x;
		ideal->y = B.y;
		ideal->z = B.z;
	}

	GetFloor(ideal->x, ideal->y, ideal->z, &ideal->room_number);
}

void ChaseCamera(ITEM_INFO *item)
{
	/* Standard follow Lara camera */
	FLOOR_INFO	*floor;
	GAME_VECTOR 	ideal;
	GAME_VECTOR	ideals[5];
	GAME_VECTOR	temp[2];
	sint32 		distance,lp,dx,dz,farthest,farthestnum;
	sint16 		angle,room_number;

	/* Limit target elevation else get horrible camera twists when nearly vertical */
	if (!camera.target_elevation)
		camera.target_elevation = NORMAL_ELEVATION;

	camera.target_elevation += item->pos.x_rot;
	if (camera.target_elevation > MAX_ELEVATION)
		camera.target_elevation = MAX_ELEVATION;
	else if (camera.target_elevation < -MAX_ELEVATION)
		camera.target_elevation = -MAX_ELEVATION;

	distance = camera.target_distance * phd_cos(camera.target_elevation) >> W2V_SHIFT;
	room_number = camera.target.room_number;
	floor = GetFloor(camera.target.x, camera.target.y, camera.target.z,&room_number);
	dx = GetHeight(floor,camera.target.x, camera.target.y, camera.target.z);
	room_number = camera.target.room_number;
	floor = GetFloor(camera.target.x, camera.target.y, camera.target.z,&room_number);
	dz = GetCeiling(floor,camera.target.x, camera.target.y, camera.target.z);
	if (camera.target.y > dx)
		camera.target.y = dx-256;
	else if (camera.target.y < dz)
		camera.target.y = dz+256;

	for (lp=0;lp<5;lp++)
		ideals[lp].y = camera.target.y + (camera.target_distance * phd_sin(camera.target_elevation) >> W2V_SHIFT);

	/* Used in ShiftCamera() */
	camera.target_square = SQUARE(distance);

	farthest = 0x7fffffff;
	farthestnum = 0;
	/* Place camera according to preset angles */
	for (lp=0;lp<5;lp++)
	{
		if (lp==0)
		{
			angle = item->pos.y_rot + camera.target_angle;
			ideals[lp].x = camera.target.x - ((distance * phd_sin(angle)) >> W2V_SHIFT);
			ideals[lp].z = camera.target.z - ((distance * phd_cos(angle)) >> W2V_SHIFT);
			ideals[lp].room_number = camera.target.room_number;
		}
		else
		{
			angle = (lp-1) * 0x4000;
			ideals[lp].x = camera.target.x - ((distance * phd_sin(angle)) >> W2V_SHIFT);
			ideals[lp].z = camera.target.z - ((distance * phd_cos(angle)) >> W2V_SHIFT);
			ideals[lp].room_number = camera.target.room_number;
		}

		if (mgLOS(&camera.target,&ideals[lp]))
		{
			if (lp == 0)
			{
				farthestnum = 0;
				break;
			}

			dx = SQUARE(camera.pos.x - ideals[lp].x);
			dx += SQUARE(camera.pos.z - ideals[lp].z);

			if (dx < farthest)
			{
				farthest = dx;
				farthestnum = lp;
			}
		}
		else if (lp==0)
		{
			dx = SQUARE(camera.target.x - ideals[lp].x);
			dz = SQUARE(camera.target.z - ideals[lp].z);

			if ((dx+dz) > SQUARE(1024))
			{
				farthestnum = 0;
				break;
			}
		}
	}

	ideal.x = ideals[farthestnum].x;
	ideal.y = ideals[farthestnum].y;
	ideal.z = ideals[farthestnum].z;
	ideal.room_number = ideals[farthestnum].room_number;

	CameraCollisionBounds(&ideal, 384);

	if (camera.old_type == FIXED_CAMERA)
		camera.speed = 1;
	MoveCamera(&ideal, camera.speed);
}

int ShiftClamp(GAME_VECTOR *pos, sint32 clamp)
{
	/* Check the position vector: if in a bad position, then clamp it to nearest allowable
		position. Doesn't do y coord, but returns the required shift instead (makes sense in
		the context of the calling function) */
	sint32 height, ceiling, shift, x, y, z;
	sint32 left, right, top, bottom;
	FLOOR_INFO *floor;
	BOX_INFO *box;

	x = pos->x;
	y = pos->y;
	z = pos->z;

	floor = GetFloor(x, y, z, &pos->room_number);

	/* Clamp to this box? */
	box = &boxes[floor->box];
	left = ((sint32)box->left << WALL_SHIFT) + clamp;
	right = ((sint32)box->right << WALL_SHIFT) - clamp - 1;
	if (z < left && !GoodPosition(x, y, z - clamp, pos->room_number))
		pos->z = left;
	else if (z > right && !GoodPosition(x, y, z + clamp, pos->room_number))
		pos->z = right;

	top = ((sint32)box->top << WALL_SHIFT) + clamp;
	bottom = ((sint32)box->bottom << WALL_SHIFT) - clamp - 1;
	if (x < top && !GoodPosition(x - clamp, y, z, pos->room_number))
		pos->x = top;
	else if (x > bottom && !GoodPosition(x + clamp, y, z, pos->room_number))
		pos->x = bottom;

	height = GetHeight(floor, x, y, z) - clamp;
	ceiling = GetCeiling(floor, x, y, z) + clamp;

	if (height < ceiling)
		height = ceiling = (height + ceiling) >> 1;

	/* Push up/down if floor/ceiling in the way */
	if (y > height)
		shift = height - y;
	else if (pos->y < ceiling)
		shift = ceiling - y;
	else
		shift = 0;

	return (shift);
}

void CombatCamera(ITEM_INFO *item)
{
	/* Standard follow Lara camera */
	FLOOR_INFO	*floor;
	GAME_VECTOR 	ideal;
	GAME_VECTOR	ideals[9];
	GAME_VECTOR	temp[2];
	sint32 		distance,lp,dx,dz,farthest,farthestnum;
	sint16 		angle,room_number;

	camera.target.z = item->pos.z_pos;
	camera.target.x = item->pos.x_pos;

	/* If Target Selected Look At It Else uses Lara's head position facing */
	if ( lara.target )
	{
		camera.target_angle = item->pos.y_rot + lara.target_angles[0];
		camera.target_elevation = item->pos.x_rot + lara.target_angles[1];
	}
	else
	{
		camera.target_angle = item->pos.y_rot + lara.head_y_rot + lara.torso_y_rot;
		camera.target_elevation = item->pos.x_rot + lara.head_x_rot + lara.torso_x_rot + NORMAL_ELEVATION;
	}

	camera.target_distance = COMBAT_DISTANCE;

	distance = camera.target_distance * phd_cos(camera.target_elevation) >> W2V_SHIFT;
	room_number = camera.target.room_number;
	floor = GetFloor(camera.target.x, camera.target.y, camera.target.z,&room_number);
	dx = GetHeight(floor,camera.target.x, camera.target.y, camera.target.z);
	room_number = camera.target.room_number;
	floor = GetFloor(camera.target.x, camera.target.y, camera.target.z,&room_number);
	dz = GetCeiling(floor,camera.target.x, camera.target.y, camera.target.z);
	if (camera.target.y > dx)
		camera.target.y = dx-256;
	else if (camera.target.y < dz)
		camera.target.y = dz+256;

	for (lp=0;lp<5;lp++)
		ideals[lp].y = camera.target.y + (camera.target_distance * phd_sin(camera.target_elevation) >> W2V_SHIFT);

	farthest = 0x7fffffff;
	farthestnum = 0;
	/* Place camera according to preset angles */
	for (lp=0;lp<5;lp++)
	{
		if (lp==0)
		{
			angle = camera.target_angle;
			ideals[lp].x = camera.target.x - ((distance * phd_sin(angle)) >> W2V_SHIFT);
			ideals[lp].z = camera.target.z - ((distance * phd_cos(angle)) >> W2V_SHIFT);
			ideals[lp].room_number = camera.target.room_number;
		}
		else
		{
			angle = (lp-1) * 0x4000;
			ideals[lp].x = camera.target.x - ((distance * phd_sin(angle)) >> W2V_SHIFT);
			ideals[lp].z = camera.target.z - ((distance * phd_cos(angle)) >> W2V_SHIFT);
			ideals[lp].room_number = camera.target.room_number;
		}

		if (mgLOS(&camera.target,&ideals[lp]))
		{
			if (lp == 0)
			{
				farthestnum = 0;
				break;
			}

			dx = SQUARE(camera.pos.x - ideals[lp].x);
			dx += SQUARE(camera.pos.z - ideals[lp].z);

			if (dx < farthest)
			{
				farthest = dx;
				farthestnum = lp;
			}
		}
		else if (lp==0)
		{
			dx = SQUARE(camera.target.x - ideals[lp].x);
			dz = SQUARE(camera.target.z - ideals[lp].z);

			if ((dx+dz) > SQUARE(1024))
			{
				farthestnum = 0;
				break;
			}
		}
	}

	ideal.x = ideals[farthestnum].x;
	ideal.y = ideals[farthestnum].y;
	ideal.z = ideals[farthestnum].z;
	ideal.room_number = ideals[farthestnum].room_number;

	CameraCollisionBounds(&ideal, 384);

	if (camera.old_type == FIXED_CAMERA)
		camera.speed = 1;
	MoveCamera(&ideal, camera.speed);
}

void LookCamera(ITEM_INFO *item)
{
	/* Looks in direction of Lara's head. If LOOK_CAMERA, shifts forwards and backwards
		when look up/down; if COMBAT_CAMERA doesn't do this */
	GAME_VECTOR 	oldt, oldp;
	PHD_VECTOR	pos1,pos2,pos3;
	FLOOR_INFO 	*floor;
	sint32		shake,lp;
	sint16 		room_number;
	sint32 		dx,dy,dz,wx,wy,wz,clipped,h,c,hxrot,txrot,hyrot,tyrot,speed,cdiff,nc;

// Pos1 = position of head, 2 = position of camera, 3 = position of target.

	hxrot = lara.head_x_rot;
	hyrot = lara.head_y_rot;
	txrot = lara.torso_x_rot;
	tyrot = lara.torso_y_rot;

	lara.torso_x_rot=0;
	lara.torso_y_rot=0;
	lara.head_x_rot <<= 1;
	lara.head_y_rot <<= 1;

	if (lara.head_x_rot > MIN_LOOKX_ROTATION)	// Look up.
		lara.head_x_rot = MIN_LOOKX_ROTATION;
	else if (lara.head_x_rot < MAX_LOOKX_ROTATION)	// Look down.
		lara.head_x_rot = MAX_LOOKX_ROTATION;
	if (lara.head_y_rot < -MAX_LOOKY_ROTATION)
		lara.head_y_rot = -MAX_LOOKY_ROTATION;
	else if (lara.head_y_rot > MAX_LOOKY_ROTATION)
		lara.head_y_rot = MAX_LOOKY_ROTATION;

	pos1.x = 0;
	pos1.y = 16;
	pos1.z = 32;
	GetLaraHandAbsPosition( &pos1, LARA_HEAD );     // Get Position of Head

	clipped = 0;
	room_number = lara_item->room_number;
	floor = GetFloor(pos1.x, pos1.y, pos1.z, &room_number);
	h=GetHeight(floor,pos1.x,pos1.y,pos1.z);
	if (pos1.y > h || h == NO_HEIGHT)
		clipped=1;
	else
	{
		room_number = lara_item->room_number;
		floor = GetFloor(pos1.x, pos1.y, pos1.z, &room_number);
		c=GetCeiling(floor,pos1.x,pos1.y,pos1.z);
		if (pos1.y < c || c == NO_HEIGHT)
			clipped=1;
	}

	if (clipped)
	{
		pos1.x = 0;
		pos1.y = 16;
		pos1.z = 0;
		GetLaraHandAbsPosition( &pos1, LARA_HEAD );     // Get Position of Head

		clipped = 0;
		room_number = lara_item->room_number;
		floor = GetFloor(pos1.x, pos1.y, pos1.z, &room_number);
		h=GetHeight(floor,pos1.x,pos1.y,pos1.z);
		if (pos1.y > h || h == NO_HEIGHT)
			clipped=1;
		else
		{
			room_number = lara_item->room_number;
			floor = GetFloor(pos1.x, pos1.y, pos1.z, &room_number);
			c=GetCeiling(floor,pos1.x,pos1.y,pos1.z);
			if (pos1.y < c || c == NO_HEIGHT)
				clipped=1;
		}

		if (clipped)
		{
			pos1.x = 0;
			pos1.y = 16;
			pos1.z = -32;
			GetLaraHandAbsPosition( &pos1, LARA_HEAD );     // Get Position of Head

			clipped = 0;
			room_number = lara_item->room_number;
			floor = GetFloor(pos1.x, pos1.y, pos1.z, &room_number);
			h=GetHeight(floor,pos1.x,pos1.y,pos1.z);
			if (pos1.y > h || h == NO_HEIGHT)
				clipped=1;
			else
			{
				room_number = lara_item->room_number;
				floor = GetFloor(pos1.x, pos1.y, pos1.z, &room_number);
				c=GetCeiling(floor,pos1.x,pos1.y,pos1.z);
				if (pos1.y < c || c == NO_HEIGHT)
					clipped=1;
			}
		}
	}

	pos2.x = 0;
	pos2.y = 0;
	pos2.z = -1024;
	GetLaraHandAbsPosition( &pos2, LARA_HEAD );     // Get Position of camera.
	pos3.x = 0;
	pos3.y = 0;
	pos3.z = 2048;
	GetLaraHandAbsPosition( &pos3, LARA_HEAD );     // Get Position of target.

	/* Remember old position to smooth camera movement */
	oldt.x = camera.target.x;
	oldt.y = camera.target.y;
	oldt.z = camera.target.z;
	oldp.x = camera.pos.x;
	oldp.y = camera.pos.y;
	oldp.z = camera.pos.z;
	oldp.room_number = camera.pos.room_number;

	dx = (pos2.x-pos1.x) >> 3;
	dy = (pos2.y-pos1.y) >> 3;
	dz = (pos2.z-pos1.z) >> 3;
	wx = pos1.x;
	wy = pos1.y;
	wz = pos1.z;

	nc = 0;
	for (lp=0;lp<8;lp++)
	{
		room_number = lara_item->room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		h = GetHeight(floor,wx,wy,wz);
		c = GetCeiling(floor,wx,wy,wz);
		cdiff = c-h;
		if (wy > h || h == NO_HEIGHT)
			break;
		if (wy < c || c == NO_HEIGHT)
			break;
		wx+=dx;
		wy+=dy;
		wz+=dz;
	}

	if (lp < 4)
	{
  		wx = pos1.x;
		wy = pos1.y;
		wz = pos1.z;
	}
	else
	{
		wx-=dx;
		wy-=dy;
		wz-=dz;
	}

	camera.pos.room_number = lara_item->room_number;

	if (	last_rotations.x == lara.head_x_rot &&
		last_rotations.y == lara.head_y_rot &&
		camera.old_type == LOOK_CAMERA)
	{
		 wx = static_lookcamp.x;
		 wy = static_lookcamp.y;
		 wz = static_lookcamp.z;
		 pos3.x = static_lookcamt.x;
		 pos3.y = static_lookcamt.y;
		 pos3.z = static_lookcamt.z;
	}
	else
	{
		last_rotations.x = lara.head_x_rot;
		last_rotations.y = lara.head_y_rot;
		static_lookcamp.x = wx;
		static_lookcamp.y = wy;
		static_lookcamp.z = wz;
		static_lookcamt.x = pos3.x;
		static_lookcamt.y = pos3.y;
		static_lookcamt.z = pos3.z;
	}

	camera.pos.x = wx;
	camera.pos.y = wy;
	camera.pos.z = wz;
	CameraCollisionBounds(&camera.pos,224);
	wx = camera.pos.x;
	wy = camera.pos.y;
	wz = camera.pos.z;

	dx = dy = dz = 512;
	speed = 4;
	while (abs(dx) > MAX_LOOKCAMERA_MOVE || abs(dy) > 255 || abs(dz) > MAX_LOOKCAMERA_MOVE)
	{
		camera.pos.x = oldp.x + (dx = ((wx - oldp.x) / speed));
		camera.pos.y = oldp.y + (dy = ((wy - oldp.y) / speed));
		camera.pos.z = oldp.z + (dz = ((wz - oldp.z) / speed));
		speed++;
	}

	camera.target.x = oldt.x + ((pos3.x - oldt.x) >> 2);
	camera.target.y = oldt.y + ((pos3.y - oldt.y) >> 2);
	camera.target.z = oldt.z + ((pos3.z - oldt.z) >> 2);
	camera.target.room_number = lara_item->room_number;

	if (camera.bounce && camera.type == camera.old_type)
	{
		if (camera.bounce > 0)
		{
			camera.pos.y += camera.bounce;
			camera.target.y += camera.bounce;
			camera.bounce = 0;
		}
		else
		{
			/* Shake camera randomly */
			shake = ((GetRandomControl()-0x4000) * camera.bounce) >> 15;
			camera.pos.x += shake;
			camera.target.y += shake;
			shake = ((GetRandomControl()-0x4000) * camera.bounce) >> 15;
			camera.pos.y += shake;
			camera.target.y += shake;
			shake = ((GetRandomControl()-0x4000) * camera.bounce) >> 15;
			camera.pos.z += shake;
			camera.target.z += shake;
			camera.bounce += 5;
		}
	}

	CameraCollisionBounds(&camera.pos,224);

	/* Final check of room number */
	GetFloor(camera.pos.x, camera.pos.y, camera.pos.z, &camera.pos.room_number);

	{
		sint16	room_number;
		long		wx,wy,wz;

		wx = camera.pos.x;
		wy = camera.pos.y;
		wz = camera.pos.z;
		room_number = camera.pos.room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		h=GetHeight(floor,wx,wy,wz);
		room_number = camera.pos.room_number;
		floor = GetFloor(wx, wy, wz, &room_number);
		c=GetCeiling(floor,wx,wy,wz);
		if (wy < c || wy > h)
  			mgLOS(&camera.target,&camera.pos);
	}

	phd_LookAt(camera.pos.x, camera.pos.y, camera.pos.z, camera.target.x, camera.target.y, camera.target.z, 0);

	/* For stereo sound effects need to know camera facing */
#ifdef PC_VERSION
	if (camera.mike_at_lara)
	{
		camera.actual_angle=lara_item->pos.y_rot+lara.head_y_rot+lara.torso_y_rot;
		camera.mike_pos.x=lara_item->pos.x_pos;
		camera.mike_pos.y=lara_item->pos.y_pos;
		camera.mike_pos.z=lara_item->pos.z_pos;
	}
	else
#endif
	{
		camera.actual_angle = phd_atan(camera.target.z - camera.pos.z, camera.target.x - camera.pos.x);
		camera.mike_pos.x=camera.pos.x+((phd_persp*phd_sin(camera.actual_angle))>>W2V_SHIFT);
		camera.mike_pos.z=camera.pos.z+((phd_persp*phd_cos(camera.actual_angle))>>W2V_SHIFT);
		camera.mike_pos.y=camera.pos.y;
	}

	camera.old_type = camera.type;

	lara.head_x_rot = hxrot;
	lara.head_y_rot = hyrot;
	lara.torso_x_rot = txrot;
	lara.torso_y_rot = tyrot;

}

void FixedCamera(void)
{
	/* Move to fixed camera position */
	GAME_VECTOR ideal;
	OBJECT_VECTOR *fixed;

	fixed = &camera.fixed[camera.number];
	ideal.x = fixed->x;
	ideal.y = fixed->y;
	ideal.z = fixed->z;
	ideal.room_number = fixed->data;

	/* If something gets between target and fixed camera, need to clip it */
	if (!LOS(&camera.target, &ideal))
		ShiftClamp(&ideal, STEP_L);

	camera.fixed_camera = 1;

	MoveCamera(&ideal, camera.speed);

	/* Deal with timer (if any) */
	if (camera.timer)
	{
		camera.timer--;
		if (!camera.timer)
			camera.timer = -1;
	}
}

void CalculateCamera(void)
{
	/* Calculate the position of the viewpoint (based eventually on all kinds of stuff) */
	ITEM_INFO 	*item;
	OBJECT_INFO *object;
	FLOOR_INFO 	*floor;
	sint16 		*bounds, angle, tilt, change;
	sint32	 	shift, fixed_camera, y;

	oldtx = camera.target.x;
	oldty = camera.target.y;
	oldtz = camera.target.z;

	if (UseForcedFixedCamera)
	{
		camera.type = FIXED_CAMERA;
		camera.pos.x = ForcedFixedCamera.x;
		camera.pos.y = ForcedFixedCamera.y;
		camera.pos.z = ForcedFixedCamera.z;
		camera.pos.room_number = ForcedFixedCamera.room_number;
	}

	if (room[lara_item->room_number].flags & SWAMP)	// If Lara is in swamp room, elevate the camera to avoid harsh polygon chaffage.
		camera.min_ypos = room[lara_item->room_number].y - 1024;
	else
		camera.min_ypos = NO_MINY;

   /* Make nice underWater sound if camera underWater */
  	if (room[camera.pos.room_number].flags & UNDERWATER)
  	{
  		SoundEffect( 60, NULL, SFX_ALWAYS );
		if (!camera.underwater)
		{
			S_CDVolume(0);
			camera.underwater = 1;
		}
  	}
	else
	{
		if (camera.underwater)
		{
	     	//mn_stop_sound_effect(60, NULL);
			//SOUND_Stop();
			if (Option_Music_Volume)
				S_CDVolume(Option_Music_Volume * 25 + 5);
			camera.underwater = 0;
		}
	}

	if (camera.type == CINEMATIC_CAMERA)
 	{
 		InGameCinematicCamera();
 		return;
	}

	/* Alter the behaviour of the GetHeight/Ceiling routines */
	if (camera.flags != NO_CHUNKY)
		chunky_flag = 0;

	/* Target may not be Lara if it has been set and fixed camera in operation */
	fixed_camera = (camera.item && (camera.type == FIXED_CAMERA || camera.type == HEAVY_CAMERA));
	item = fixed_camera ? camera.item : lara_item;

	/* Setup target (which is item's mid point filtered to remove excess shake) Ha! Of course it does. */
	object = &objects[item->object_number];
	bounds = GetBoundsAccurate(item);

	y = item->pos.y_pos;
	if (!fixed_camera)
		y += bounds[3] + ((bounds[2] - bounds[3]) * 3 >> 2);
	else
		y += (bounds[2] + bounds[3])/2;

	/* TARGET_CAMERA: If target but no camera, then make Lara's head turn to look at it */
	if (camera.item && !fixed_camera)
	{
		shift = phd_sqrt(SQUARE(camera.item->pos.z_pos - item->pos.z_pos) + SQUARE(camera.item->pos.x_pos - item->pos.x_pos));
		angle = phd_atan(camera.item->pos.z_pos - item->pos.z_pos, camera.item->pos.x_pos - item->pos.x_pos) - item->pos.y_rot;
		bounds = GetBoundsAccurate(camera.item);
		tilt = phd_atan(shift, y - (camera.item->pos.y_pos + (bounds[2] + bounds[3])/2));
		angle >>= 1;
		tilt >>= 1;
		if (angle > -MAX_HEAD_ROTATION && angle < MAX_HEAD_ROTATION && tilt > MIN_HEAD_TILT && tilt < MAX_HEAD_TILT)
		{
			change = angle - lara.head_y_rot;
			if (change > HEAD_TURN)
				lara.head_y_rot += HEAD_TURN;
			else if (change < -HEAD_TURN)
				lara.head_y_rot -= HEAD_TURN;
			else
				lara.head_y_rot += change;
			lara.torso_y_rot = lara.head_y_rot;

			change = tilt - lara.head_x_rot;
			if (change > HEAD_TURN)
				lara.head_x_rot += HEAD_TURN;
			else if (change < -HEAD_TURN)
				lara.head_x_rot -= HEAD_TURN;
			else
				lara.head_x_rot += change;
			lara.torso_x_rot = lara.head_x_rot;

			camera.type = LOOK_CAMERA;

			/* Flag that this item has been 'spotted' by Lara */
			camera.item->looked_at = 1;
		}
	}

	if (camera.type == LOOK_CAMERA || camera.type == COMBAT_CAMERA)
	{
		/* To do the midly unpleasant target shift when look up down, need to know old position */
		y -= STEP_L;
		camera.target.room_number = item->room_number;

		/* Don't smooth target shift if coming from FIXED_CAMERA/HEAVY_CAMERA */
		if (camera.fixed_camera)
		{
			camera.target.y = y;
			camera.speed = 1;
		}
		else
		{
			camera.target.y += ((y - camera.target.y) >> 2);
			camera.speed = (camera.type == LOOK_CAMERA)? LOOK_SPEED : COMBAT_SPEED;
		}

		/* Fixed camera allows speed to be set for camera movement (as well as remembering if last
			camera was fixed with non-Lara target) */
		camera.fixed_camera = 0;

		if (camera.type == LOOK_CAMERA)
			LookCamera(item);
		else
			CombatCamera(item);
	}
	else
	{
		/* Shift to follow centre of bounds if required (e.g. Push/PullBlock) */

		camera.target.x = item->pos.x_pos;
		camera.target.z = item->pos.z_pos;

		if (camera.flags == FOLLOW_CENTRE)
		{
			shift = (bounds[4] + bounds[5])/2;
			camera.target.z += phd_cos(item->pos.y_rot) * shift >> W2V_SHIFT;
			camera.target.x += phd_sin(item->pos.y_rot) * shift >> W2V_SHIFT;
		}
		camera.target.room_number = item->room_number;

		/* Don't smooth target shift if switching between fixed cameras (on or off) */
		if (camera.fixed_camera ^ fixed_camera)
		{
			camera.target.y = y;
			camera.fixed_camera = 1;
			camera.speed = 1;
		}
		else
		{
			//camera.target.y += (y - camera.target.y)/4;
			camera.target.y = y;
			camera.fixed_camera = 0;
		}

		if (camera.speed != 1 && camera.old_type != LOOK_CAMERA)
		{
			camera.target.x = oldtx + ((camera.target.x - oldtx) >> 2);
			camera.target.y = oldty + ((camera.target.y - oldty) >> 2);
			camera.target.z = oldtz + ((camera.target.z - oldtz) >> 2);
		}

		/* Possible for item's centre point to be shifted into another room */
		floor = GetFloor( camera.target.x, camera.target.y, camera.target.z, &camera.target.room_number);

		/* A target on a steep slope may cause camera to go mad if chunky GetHeight is used */
		if (camera.target.y > GetHeight(floor, camera.target.x, camera.target.y, camera.target.z))
			chunky_flag = 0;

		if (camera.type == CHASE_CAMERA || camera.flags == CHASE_OBJECT)
			ChaseCamera(item);
		else
			FixedCamera();
	}

	camera.last = camera.number;
	camera.fixed_camera = fixed_camera;

	if (camera.type != HEAVY_CAMERA || camera.timer == -1)
	{
		/* Reset camera stuff for next time */
		camera.type = CHASE_CAMERA;
		camera.speed = CHASE_SPEED;
		camera.number = NO_CAMERA;
		camera.last_item = camera.item;
		camera.item = NULL;
		camera.target_angle = camera.target_elevation = 0;
		camera.target_distance = WALL_L*3/2;
		camera.flags = 0;
	}

	chunky_flag = 0;
}


long	CameraCollisionBounds(GAME_VECTOR *ideal, long push)
{
	FLOOR_INFO *floor;
	long		sx,sy,sz;
	long		wx,wy,wz,dy,clipdy,go_again;
	sint16 	room_number;

	wx = ideal->x;
	wy = ideal->y;
	wz = ideal->z;

	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz, &room_number);
	clipdy=GetHeight(floor,wx,wy,wz);
	dy=GetCeiling(floor,wx,wy,wz);
	if (wy-255 < dy && wy+255 > clipdy && abs(wy-dy) < 16384)
		wy = (clipdy+dy) >> 1;
	else if (wy+255 > clipdy && abs(wy-dy) < 16384)
		wy = clipdy-255;
	else if (wy-255 < dy && abs(wy-dy) < 16384)
		wy = dy+255;

	room_number = ideal->room_number;
	floor = GetFloor(wx-push, wy, wz, &room_number);
	dy=GetHeight(floor,wx-push,wy,wz);
	if (wy > dy || dy == NO_HEIGHT)
		wx = (wx&(~1023))+push;
	room_number = ideal->room_number;
	floor = GetFloor(wx-push, wy, wz, &room_number);
	dy=GetCeiling(floor,wx-push,wy,wz);
	if (wy < dy || dy == NO_HEIGHT)
		wx = (wx&(~1023))+push;

	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz-push, &room_number);
	dy=GetHeight(floor,wx,wy,wz-push);
	if (wy > dy || dy == NO_HEIGHT)
		wz = (wz&(~1023))+push;
	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz-push, &room_number);
	dy=GetCeiling(floor,wx,wy,wz-push);
	if (wy < dy || dy == NO_HEIGHT)
		wz = (wz&(~1023))+push;

	room_number = ideal->room_number;
	floor = GetFloor(wx+push, wy, wz, &room_number);
	dy=GetHeight(floor,wx+push,wy,wz);
	if (wy > dy || dy == NO_HEIGHT)
		wx = (wx|1023)-push;
	room_number = ideal->room_number;
	floor = GetFloor(wx+push, wy, wz, &room_number);
	dy=GetCeiling(floor,wx+push,wy,wz);
	if (wy < dy || dy == NO_HEIGHT)
		wx = (wx|1023)-push;

	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz+push, &room_number);
	dy=GetHeight(floor,wx,wy,wz+push);
	if (wy > dy || dy == NO_HEIGHT)
		wz = (wz|1023)-push;
	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz+push, &room_number);
	dy=GetCeiling(floor,wx,wy,wz+push);
	if (wy < dy || dy == NO_HEIGHT)
		wz = (wz|1023)-push;

	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz, &room_number);
	clipdy=GetHeight(floor,wx,wy,wz);
	room_number = ideal->room_number;
	floor = GetFloor(wx, wy, wz, &room_number);
	dy=GetCeiling(floor,wx,wy,wz);
	if (wy < dy || wy > clipdy)
		return(1);

	floor = GetFloor(wx, wy, wz, &ideal->room_number);
	ideal->x = wx;
	ideal->y = wy;
	ideal->z = wz;

	return (0);
}

long mgLOS(GAME_VECTOR *start, GAME_VECTOR *target)
{
	FLOOR_INFO *floor;
	long		x,y,z,h,c,cdiff;
	long		dx,dy,dz,lp,clipped,nc;
	sint16 	room_number;

	dx = (target->x - start->x)>>3;
	dy = (target->y - start->y)>>3;
	dz = (target->z - start->z)>>3;

	x = start->x;
	y = start->y;
	z = start->z;
	room_number = start->room_number;

	clipped = nc = 0;
	for (lp=0;lp<8;lp++)
	{
		floor = GetFloor(x, y, z, &room_number);
		h = GetHeight(floor, x, y, z);
		c = GetCeiling(floor, x, y, z);
		cdiff = c-h;
		if (y > h || h == NO_HEIGHT)
		{
 			clipped = 1;
 			break;
		}

		if (y < c || c == NO_HEIGHT)
		{
 			clipped = 1;
 			break;
		}

		x += dx;
		y += dy;
		z += dz;
	}

	x -= dx;
	y -= dy;
	z -= dz;

	target->x = x;
	target->y = y;
	target->z = z;

	GetFloor(target->x, target->y, target->z, &room_number);
	target->room_number = room_number;

	return (!clipped);
}
