/**************************************************************************************************************/
/*                                                                                                            */
/* Underwater creature control																				  	  G.Rummery 1997 */
/*                                                                                                            */
/**************************************************************************************************************/

/*---------------------------------------------------------------------------
 *	Import Headers
\*--------------------------------------------------------------------------*/

#ifdef PSX_VERSION
#include "../spec_psx/typedefs.h"
#else
#include "../specific/stypes.h"
#endif

#include "objects.h"
#include "lara.h"
#include "laraanim.h"
#include "control.h"

/*---------------------------------------------------------------------------
 *	Constants
\*--------------------------------------------------------------------------*/

#define SHARK_BITE 400
#define SHARK_TOUCH 0x3400

#define SHARK_SWIM2_RANGE SQUARE(WALL_L*3)
#define SHARK_ATTACK1_RANGE SQUARE(WALL_L*3/4)
#define SHARK_ATTACK2_RANGE SQUARE(WALL_L*4/3)

#define SHARK_SWIM1_TURN (ONE_DEGREE/2)
#define SHARK_SWIM2_TURN (ONE_DEGREE*2)

#define SHARK_ATTACK1_CHANCE 0x800

#define SHARK_DIE_ANIM 4
#define SHARK_KILL_ANIM 19

enum shark_anim {
	SHARK_STOP,
	SHARK_SWIM1,
	SHARK_SWIM2,
	SHARK_ATTACK1,
	SHARK_ATTACK2,
	SHARK_DEATH,
	SHARK_KILL
};

/*---------------------------------------------------------------------------
 *	Locals
\*--------------------------------------------------------------------------*/

static BITE_INFO shark_bite = { 17, -22, 344, 12 };

/*---------------------------------------------------------------------------
 *	Public Functions
\*--------------------------------------------------------------------------*/

void SharkControl(sint16 item_number)
{
	ITEM_INFO *item;
	CREATURE_INFO *shark;
	sint16 angle, head;
	AI_INFO info;
	int lara_alive;
//sint16 *bounds;

	if (!CreatureActive(item_number))
		return;

	item = &items[item_number];
	shark = (CREATURE_INFO *)item->data;
	angle = head = 0;

	lara_alive = (lara_item->hit_points>0);

	if (item->hit_points <= 0)
	{
		if (item->current_anim_state != SHARK_DEATH)
		{
			item->anim_number = objects[WHALE].anim_index + SHARK_DIE_ANIM;
			item->frame_number = anims[item->anim_number].frame_base;
			item->current_anim_state = SHARK_DEATH;
		}

		/* Make dead shark float to surface */
		CreatureFloat(item_number);

		return;
	}
	else
	{
		CreatureAIInfo(item, &info);

		CreatureMood(item, &info, VIOLENT);

		angle = CreatureTurn(item, shark->maximum_turn);

		switch (item->current_anim_state)
		{
		case SHARK_STOP:
			/* flags mark the attack status */
			shark->flags = 0;
			shark->maximum_turn = 0;

			if (info.ahead && info.distance < SHARK_ATTACK1_RANGE && info.zone_number==info.enemy_zone)
				item->goal_anim_state = SHARK_ATTACK1;
			else
				item->goal_anim_state = SHARK_SWIM1;
			break;

		case SHARK_SWIM1:
			shark->maximum_turn = SHARK_SWIM1_TURN;

			if (shark->mood == BORED_MOOD)
				break;
			else if (info.ahead && info.distance < SHARK_ATTACK1_RANGE)
				item->goal_anim_state = SHARK_STOP;
			else if (shark->mood == ESCAPE_MOOD || info.distance > SHARK_SWIM2_RANGE || !info.ahead)
				item->goal_anim_state = SHARK_SWIM2;
			break;

		case SHARK_SWIM2:
			shark->flags = 0;
			shark->maximum_turn = SHARK_SWIM2_TURN;

			if (shark->mood == BORED_MOOD)
				item->goal_anim_state = SHARK_SWIM1;
			else if (shark->mood == ESCAPE_MOOD)
				break;
			else if (info.ahead && info.distance < SHARK_ATTACK2_RANGE && info.zone_number==info.enemy_zone)
			{
				if (GetRandomControl() < SHARK_ATTACK1_CHANCE)
					item->goal_anim_state = SHARK_STOP;
				else if (info.distance < SHARK_ATTACK1_RANGE)
					item->goal_anim_state = SHARK_ATTACK2;
			}
			break;

		case SHARK_ATTACK1:
		case SHARK_ATTACK2:
			if (info.ahead)
				head = info.angle;

			if (!shark->flags && (item->touch_bits & SHARK_TOUCH))
			{
				lara_item->hit_points -= SHARK_BITE;
				lara_item->hit_status = 1;
				CreatureEffect(item, &shark_bite, DoBloodSplat);

				shark->flags = 1;
			}
			break;
		}
	}

	/* Is Lara dead? We've killed her! */
	if (lara_alive && lara_item->hit_points <= 0)
	{
		CreatureKill(item, SHARK_KILL_ANIM, SHARK_KILL, EXTRA_SHARKKILL);
		return;
	}

//bounds = GetBestFrame(item);
//Log("%d:%d Bounds%d,%d %d,%d %d,%d", item->current_anim_state, item->frame_number, bounds[0], bounds[1], bounds[2], bounds[3], bounds[4], bounds[5]);

	if (item->current_anim_state != SHARK_KILL)
	{
//		CreatureHead(item, head);

		CreatureAnimation(item_number, angle, 0);

		/* Don't allow shark to get above the water surface */
		CreatureUnderwater(item, WALL_L/3);
	}
	else
		AnimateItem(item);
}
