

/*---------------------------------------------------------------------------
 *	Import Headers
\*--------------------------------------------------------------------------*/

#include "typedefs.h"
#include "3d_gen.h"

#undef phd_mxptr
#undef w2v_matrix
#undef matrix_stack

/*---------------------------------------------------------------------------
 *	Constants
\*--------------------------------------------------------------------------*/

#define MAX_MATRICES 2	// 50

/*---------------------------------------------------------------------------
 *	Globals
\*--------------------------------------------------------------------------*/

sint32 phd_left;		// Left Clipping Value
sint32 phd_right;		// Right Clipping Value
sint32 phd_top;		// Top Clipping Value
sint32 phd_bottom;		// Bottom Clipping Value
sint32 phd_persp;		// perspective thang

sint32* phd_mxptr;		// matrix pointer....
sint32 w2v_matrix[12];	// Copy Of W2V matrix...
sint32 matrix_stack[MAX_MATRICES*12];

//PHD_VECTOR ls_vector_view;		// light vector in View Frame
//PHD_ANGLE phd_ls_pitch,phd_ls_yaw;	// Angles Of Any Light Source

//sint32 phd_winxmax;		// Maximum Window X coord           // NOW DEFINED AS CONSTANTS!!! (or are they...)
//sint32 phd_winwidth;	// Window Width
//sint32 phd_centerx;		// Vanishing point X
//sint32 phd_scrwidth;	// Width of Screen In Pixels

PHD_3DPOS 	viewer;	// ##

#ifdef PSX_VERSION
VECTOR3D CamPos, CamTgt;
#endif

/*---------------------------------------------------------------------------
 *	Public Functions
\*--------------------------------------------------------------------------*/

void	phd_GenerateW2V( PHD_3DPOS *viewpos )
{
	sint32	sx,sy,sz;
	sint32	cx,cy,cz;
	sint32	*mptr,*nptr;

	mptr = phd_mxptr = matrix_stack;    /* Put W2V at Top Of matrix Stack */
	nptr = w2v_matrix;

	sx = phd_sin( viewpos->x_rot );		/* get sin and cos values*/
	cx = phd_cos( viewpos->x_rot );
	sy = phd_sin( viewpos->y_rot );
	cy = phd_cos( viewpos->y_rot );
	sz = phd_sin( viewpos->z_rot );
	cz = phd_cos( viewpos->z_rot );

	*(nptr+M00) = *(mptr+M00) = (TRIGMULT3(sx, sy, sz) + TRIGMULT2(cy, cz));
  	*(nptr+M01) = *(mptr+M01) = (TRIGMULT2(cx, sz));
  	*(nptr+M02) = *(mptr+M02) = (TRIGMULT3(sx, cy, sz) - TRIGMULT2(sy, cz));
  	*(nptr+M10) = *(mptr+M10) = (TRIGMULT3(sx, sy, cz) - TRIGMULT2(cy, sz));
  	*(nptr+M11) = *(mptr+M11) = (TRIGMULT2(cx, cz));
  	*(nptr+M12) = *(mptr+M12) = (TRIGMULT3(sx, cy, cz) + TRIGMULT2(sy, sz));
  	*(nptr+M20) = *(mptr+M20) = (TRIGMULT2(cx, sy));
  	*(nptr+M21) = *(mptr+M21) = -(sx);
  	*(nptr+M22) = *(mptr+M22) = (TRIGMULT2(cx, cy));
	*(nptr+M03) = *(mptr+M03) = viewpos->x_pos;
	*(nptr+M13) = *(mptr+M13) = viewpos->y_pos;
	*(nptr+M23) = *(mptr+M23) = viewpos->z_pos;

/* -------- fix aspect ratio */

#if 0
	*(nptr+M10)=*(mptr+M10)=*(mptr+M10)-(*(mptr+M10)>>2);
	*(nptr+M11)=*(mptr+M11)=*(mptr+M11)-(*(mptr+M11)>>2);
	*(nptr+M12)=*(mptr+M12)=*(mptr+M12)-(*(mptr+M12)>>2);
#else
	*(mptr+M10) = *(mptr+M10) - (*(mptr+M10)>>2);	// only on stack
	*(mptr+M11) = *(mptr+M11) - (*(mptr+M11)>>2);
	*(mptr+M12) = *(mptr+M12) - (*(mptr+M12)>>2);
#endif
}

/******************************************************************************
 *			Set Up W2V to look at specified Target
 *****************************************************************************/
void	phd_LookAt( sint32 xsrc, sint32 ysrc, sint32 zsrc,
					sint32 xtar, sint32 ytar, sint32 ztar,
					PHD_ANGLE roll )
{
	PHD_ANGLE	angles[2];
//##	PHD_3DPOS 	viewer;

	phd_GetVectorAngles( xtar-xsrc,ytar-ysrc,ztar-zsrc, angles );
	viewer.x_pos = xsrc;
	viewer.y_pos = ysrc;
	viewer.z_pos = zsrc;
	viewer.x_rot = angles[1];
	viewer.y_rot = angles[0];
	viewer.z_rot = roll;
	phd_GenerateW2V(&viewer);

	CamPos.x = xsrc;
	CamPos.y = ysrc;
	CamPos.z = zsrc;
}

/******************************************************************************
 *				Get Angles In 3D Space from Source to Dest
 *					get Yaw angle at pos 0
 *					  Pitch angle at pos 1
 *****************************************************************************/
void	phd_GetVectorAngles(sint32 x, sint32 y, sint32 z, PHD_ANGLE *dest)
{
	PHD_ANGLE	pitch;

	*(dest+0) = phd_atan(z, x);

	while  ((sint16)x!=x || (sint16)y!=y || (sint16)z!=z)
	{
		x >>= 2;
		y >>= 2;                /* Scale Coords Into Word Size */
		z >>= 2;
	}

	pitch = phd_atan(phd_sqrt(x*x + z*z), y);

	if ( (y>0 && pitch>0) || (y<0 && pitch<0) )
		pitch = -pitch;

	*(dest+1) = pitch;
}

/******************************************************************************
 *					Normalise A Vector To Length 16384
 *****************************************************************************/
void	phd_NormaliseVector( sint32 x, sint32 y, sint32 z, sint32 *dest )
{
	sint32	len;

	if ( x || y || z )
	{
		while  ( (sint16)x!=x || (sint16)y!=y || (sint16)z!=z )
		{
			x >>= 2;        /* Scale Coords Into Word size values..*/
			y >>= 2;
			z >>= 2;
		}
		len = phd_sqrt( x*x + y*y + z*z );
		*(dest+0) = (x<<W2V_SHIFT)/len;
		*(dest+1) = (y<<W2V_SHIFT)/len;
		*(dest+2) = (z<<W2V_SHIFT)/len;
	}
	else
	{
		*(dest+0) = 0;		/* If Vector Is Zero Length*/
		*(dest+1) = 0;      /* then return Null Vector*/
		*(dest+2) = 0;
	}
}

#if 0
/******************************************************************************
 *				Normalise And Regenerate Current Matrix
 *****************************************************************************/
void	phd_NormaliseMatrix( void )
{
	PHD_ANGLE	angles[3];

	phd_GetMatrixAngles( angles );
	phd_UnitMatrix();
	phd_RotYXZ( angles[1],angles[0],angles[2] );
}

/******************************************************************************
 *		  Get Rotation Angles From Current Matrix In ROT.ORDER ( Z,X,Y )
 *			saves in order rot_x, rot_y, rot_z
 *****************************************************************************/
void	phd_GetMatrixAngles( PHD_ANGLE *dest )
{
	PHD_ANGLE	roll,pitch,yaw;
	sint32		sy,cy;
	sint32		*mptr = phd_mxptr;

	yaw = phd_atan( *(mptr+M22), *(mptr+M02) );
	pitch = phd_atan( phd_sqrt( *(mptr+M22) * *(mptr+M22) + *(mptr+M02) * *(mptr+M02) ),
					  *(mptr+M12) );
	sy = phd_sin( yaw );
	cy = phd_cos( yaw );
	roll = phd_atan( ( (cy * *(mptr+M00)) - (sy * *(mptr+M20)) ),
					 ( (sy * *(mptr+M21)) - (cy * *(mptr+M01)) ) );
	if ( ( *(mptr+M12)>=0 && pitch>0 ) || ( *(mptr+M12)<0 && pitch<0 ) )
		pitch = -pitch;

	*(dest+0) = pitch;             /* save RotX*/
	*(dest+1) = yaw;               /* save RotY*/
	*(dest+2) = roll;              /* save RotZ*/
}


/******************************************************************************
 ******************************************************************************
 *		Rotate Current Matrix Around its X-axis
 ******************************************************************************
 *****************************************************************************/
void	phd_RotX( PHD_ANGLE rx )
{
	sint32	sx,cx,r0,r1;
	sint32	*mptr;

	if ( rx )
	{
		mptr = phd_mxptr;
		sx = phd_sin(rx);
		cx = phd_cos(rx);

		r0 = *(mptr+M01)*cx + *(mptr+M02)*sx;
		r1 = *(mptr+M02)*cx - *(mptr+M01)*sx;
		*(mptr+M01) = r0>>W2V_SHIFT;
		*(mptr+M02) = r1>>W2V_SHIFT;

		r0 = *(mptr+M11)*cx + *(mptr+M12)*sx;
		r1 = *(mptr+M12)*cx - *(mptr+M11)*sx;
		*(mptr+M11) = r0>>W2V_SHIFT;
		*(mptr+M12) = r1>>W2V_SHIFT;

		r0 = *(mptr+M21)*cx + *(mptr+M22)*sx;
		r1 = *(mptr+M22)*cx - *(mptr+M21)*sx;
		*(mptr+M21) = r0>>W2V_SHIFT;
		*(mptr+M22) = r1>>W2V_SHIFT;
	}
}

/******************************************************************************
 ******************************************************************************
 *		Rotate Current Matrix Around its Y-axis
 ******************************************************************************
 *****************************************************************************/
void	phd_RotY( PHD_ANGLE ry )
{
	sint32	sy,cy,r0,r1;
	sint32	*mptr;

	if ( ry )
	{
		mptr = phd_mxptr;
		sy = phd_sin(ry);
		cy = phd_cos(ry);

		r0 = *(mptr+M00)*cy - *(mptr+M02)*sy;
		r1 = *(mptr+M02)*cy + *(mptr+M00)*sy;
		*(mptr+M00) = r0>>W2V_SHIFT;
		*(mptr+M02) = r1>>W2V_SHIFT;

		r0 = *(mptr+M10)*cy - *(mptr+M12)*sy;
		r1 = *(mptr+M12)*cy + *(mptr+M10)*sy;
		*(mptr+M10) = r0>>W2V_SHIFT;
		*(mptr+M12) = r1>>W2V_SHIFT;

		r0 = *(mptr+M20)*cy - *(mptr+M22)*sy;
		r1 = *(mptr+M22)*cy + *(mptr+M20)*sy;
		*(mptr+M20) = r0>>W2V_SHIFT;
		*(mptr+M22) = r1>>W2V_SHIFT;
	}
}


/******************************************************************************
 ******************************************************************************
 *				Rotate Current Matrix Around its Z-axis
 ******************************************************************************
 *****************************************************************************/
void	phd_RotZ( PHD_ANGLE rz )
{
	sint32	sz,cz,r0,r1;
	sint32	*mptr;

	if ( rz )
	{
		mptr = phd_mxptr;
		sz = phd_sin(rz);
		cz = phd_cos(rz);

		r0 = *(mptr+M00)*cz + *(mptr+M01)*sz;
		r1 = *(mptr+M01)*cz - *(mptr+M00)*sz;
		*(mptr+M00) = r0>>W2V_SHIFT;
		*(mptr+M01) = r1>>W2V_SHIFT;

		r0 = *(mptr+M10)*cz + *(mptr+M11)*sz;
		r1 = *(mptr+M11)*cz - *(mptr+M10)*sz;
		*(mptr+M10) = r0>>W2V_SHIFT;
		*(mptr+M11) = r1>>W2V_SHIFT;

		r0 = *(mptr+M20)*cz + *(mptr+M21)*sz;
		r1 = *(mptr+M21)*cz - *(mptr+M20)*sz;
		*(mptr+M20) = r0>>W2V_SHIFT;
		*(mptr+M21) = r1>>W2V_SHIFT;
	}
}
#endif

#if 0
/******************************************************************************
 ******************************************************************************
 *		Do Successive Rotations In Order Y-X-Z
 ******************************************************************************
 *****************************************************************************/
void	phd_RotYXZ( PHD_ANGLE ry, PHD_ANGLE rx, PHD_ANGLE rz )
{
	sint32	sina,cosa,r0,r1;
	sint32	*mptr;

	mptr = phd_mxptr;
	if ( ry )
	{
		sina = phd_sin(ry);
		cosa = phd_cos(ry);
		r0 = *(mptr+M00)*cosa - *(mptr+M02)*sina;
		r1 = *(mptr+M02)*cosa + *(mptr+M00)*sina;
		*(mptr+M00) = r0>>W2V_SHIFT;
		*(mptr+M02) = r1>>W2V_SHIFT;
		r0 = *(mptr+M10)*cosa - *(mptr+M12)*sina;
		r1 = *(mptr+M12)*cosa + *(mptr+M10)*sina;
		*(mptr+M10) = r0>>W2V_SHIFT;
		*(mptr+M12) = r1>>W2V_SHIFT;
		r0 = *(mptr+M20)*cosa - *(mptr+M22)*sina;
		r1 = *(mptr+M22)*cosa + *(mptr+M20)*sina;
		*(mptr+M20) = r0>>W2V_SHIFT;
		*(mptr+M22) = r1>>W2V_SHIFT;
	}
	if ( rx )
	{
		sina = phd_sin(rx);
		cosa = phd_cos(rx);
		r0 = *(mptr+M01)*cosa + *(mptr+M02)*sina;
		r1 = *(mptr+M02)*cosa - *(mptr+M01)*sina;
		*(mptr+M01) = r0>>W2V_SHIFT;
		*(mptr+M02) = r1>>W2V_SHIFT;
		r0 = *(mptr+M11)*cosa + *(mptr+M12)*sina;
		r1 = *(mptr+M12)*cosa - *(mptr+M11)*sina;
		*(mptr+M11) = r0>>W2V_SHIFT;
		*(mptr+M12) = r1>>W2V_SHIFT;
		r0 = *(mptr+M21)*cosa + *(mptr+M22)*sina;
		r1 = *(mptr+M22)*cosa - *(mptr+M21)*sina;
		*(mptr+M21) = r0>>W2V_SHIFT;
		*(mptr+M22) = r1>>W2V_SHIFT;
	}
	if ( rz )
	{
		sina = phd_sin(rz);
		cosa = phd_cos(rz);
		r0 = *(mptr+M00)*cosa + *(mptr+M01)*sina;
		r1 = *(mptr+M01)*cosa - *(mptr+M00)*sina;
		*(mptr+M00) = r0>>W2V_SHIFT;
		*(mptr+M01) = r1>>W2V_SHIFT;
		r0 = *(mptr+M10)*cosa + *(mptr+M11)*sina;
		r1 = *(mptr+M11)*cosa - *(mptr+M10)*sina;
		*(mptr+M10) = r0>>W2V_SHIFT;
		*(mptr+M11) = r1>>W2V_SHIFT;
		r0 = *(mptr+M20)*cosa + *(mptr+M21)*sina;
		r1 = *(mptr+M21)*cosa - *(mptr+M20)*sina;
		*(mptr+M20) = r0>>W2V_SHIFT;
		*(mptr+M21) = r1>>W2V_SHIFT;
	}
}

/******************************************************************************
 ******************************************************************************
 *		Do Successive Rotations In Order Y-X-Z
 *			Rotations Are packed In Long Word.. 10 Bits Each
 *				RotX = Bits 29..20
 *				RotY = Bits 19..10
 *				RotZ = Bits 09..00
 ******************************************************************************
 *****************************************************************************/
void	phd_RotYXZpack( sint32	rots )
{
	sint32		ang;
	sint32		sina,cosa,r0,r1;
	sint32		*mptr;

	mptr = phd_mxptr;

	ang = (((rots>>10)&1023)<<6);		// Extract RotY

	if ( ang )
	{
		sina = phd_sin(ang);
		cosa = phd_cos(ang);
		r0 = *(mptr+M00)*cosa - *(mptr+M02)*sina;
		r1 = *(mptr+M02)*cosa + *(mptr+M00)*sina;
		*(mptr+M00) = r0>>W2V_SHIFT;
		*(mptr+M02) = r1>>W2V_SHIFT;
		r0 = *(mptr+M10)*cosa - *(mptr+M12)*sina;
		r1 = *(mptr+M12)*cosa + *(mptr+M10)*sina;
		*(mptr+M10) = r0>>W2V_SHIFT;
		*(mptr+M12) = r1>>W2V_SHIFT;
		r0 = *(mptr+M20)*cosa - *(mptr+M22)*sina;
		r1 = *(mptr+M22)*cosa + *(mptr+M20)*sina;
		*(mptr+M20) = r0>>W2V_SHIFT;
		*(mptr+M22) = r1>>W2V_SHIFT;
	}
	ang = (((rots>>20)&1023)<<6);		// Extract RotX
	if ( ang )
	{
		sina = phd_sin(ang);
		cosa = phd_cos(ang);
		r0 = *(mptr+M01)*cosa + *(mptr+M02)*sina;
		r1 = *(mptr+M02)*cosa - *(mptr+M01)*sina;
		*(mptr+M01) = r0>>W2V_SHIFT;
		*(mptr+M02) = r1>>W2V_SHIFT;
		r0 = *(mptr+M11)*cosa + *(mptr+M12)*sina;
		r1 = *(mptr+M12)*cosa - *(mptr+M11)*sina;
		*(mptr+M11) = r0>>W2V_SHIFT;
		*(mptr+M12) = r1>>W2V_SHIFT;
		r0 = *(mptr+M21)*cosa + *(mptr+M22)*sina;
		r1 = *(mptr+M22)*cosa - *(mptr+M21)*sina;
		*(mptr+M21) = r0>>W2V_SHIFT;
		*(mptr+M22) = r1>>W2V_SHIFT;
	}
	ang = ((rots&1023)<<6);				// Extract RotZ
	if ( ang )
	{
		sina = phd_sin(ang);
		cosa = phd_cos(ang);
		r0 = *(mptr+M00)*cosa + *(mptr+M01)*sina;
		r1 = *(mptr+M01)*cosa - *(mptr+M00)*sina;
		*(mptr+M00) = r0>>W2V_SHIFT;
		*(mptr+M01) = r1>>W2V_SHIFT;
		r0 = *(mptr+M10)*cosa + *(mptr+M11)*sina;
		r1 = *(mptr+M11)*cosa - *(mptr+M10)*sina;
		*(mptr+M10) = r0>>W2V_SHIFT;
		*(mptr+M11) = r1>>W2V_SHIFT;
		r0 = *(mptr+M20)*cosa + *(mptr+M21)*sina;
		r1 = *(mptr+M21)*cosa - *(mptr+M20)*sina;
		*(mptr+M20) = r0>>W2V_SHIFT;
		*(mptr+M21) = r1>>W2V_SHIFT;
	}
}
#endif

#ifndef USE_ASM
/******************************************************************************
 ******************************************************************************
 *				Translate Matrix to Relative Position
 ******************************************************************************
 *****************************************************************************/
int		phd_TranslateRel( sint32 x, sint32 y, sint32 z )
{
	sint32	*mptr;

	mptr = phd_mxptr;
	*(mptr+M03) += *(mptr+M00)*x + *(mptr+M01)*y + *(mptr+M02)*z;
	*(mptr+M13) += *(mptr+M10)*x + *(mptr+M11)*y + *(mptr+M12)*z;
	*(mptr+M23) += *(mptr+M20)*x + *(mptr+M21)*y + *(mptr+M22)*z;
	if ( (x = ABS(*(mptr+M03))) > phd_zfar ||
		 (y = ABS(*(mptr+M13))) > phd_zfar ||
		 (z = ABS(*(mptr+M23))) > phd_zfar )
		return(0);
	return(1);
}

/******************************************************************************
 ******************************************************************************
 *				Translate Matrix to Absolute Position
 *					returns 0 if out of range!!!
 ******************************************************************************
 *****************************************************************************/
void	phd_TranslateAbs( sint32 x, sint32 y, sint32 z )
{
	sint32	*mptr = phd_mxptr;

	x -= *(w2v_matrix+M03);          /* Subtract Position Of */
	y -= *(w2v_matrix+M13);          /* W2V Matrix  */
	z -= *(w2v_matrix+M23);
	*(mptr+M03) = *(mptr+M00)*x + *(mptr+M01)*y + *(mptr+M02)*z;
	*(mptr+M13) = *(mptr+M10)*x + *(mptr+M11)*y + *(mptr+M12)*z;
	*(mptr+M23) = *(mptr+M20)*x + *(mptr+M21)*y + *(mptr+M22)*z;
}
#endif


/*****************************************************************************
 *		Position Global Light Vector In World Using Angles
 ****************************************************************************/
#if 0
void	phd_RotateLight( PHD_ANGLE pitch, PHD_ANGLE yaw )
{
	PHD_VECTOR	ls_world;
	sint32		*mptr;
	sint32		cx;

	phd_ls_yaw = yaw;
	phd_ls_pitch = pitch;

	cx = (sint32)phd_cos(pitch);
	ls_world.x = TRIGMULT2( cx, (sint32)phd_sin(yaw) );	  // Calc World Frame
	ls_world.y = -phd_sin(pitch);                         // Light Vector
	ls_world.z = TRIGMULT2( cx, (sint32)phd_cos(yaw) );

	mptr = w2v_matrix;
	ls_vector_view.x = ( *(mptr+M00) * ls_world.x		  // Rotate Into
					   + *(mptr+M01) * ls_world.y         // View Frame..
					   + *(mptr+M02) * ls_world.z ) >> W2V_SHIFT;
	ls_vector_view.y = ( *(mptr+M10) * ls_world.x
					   + *(mptr+M11) * ls_world.y
					   + *(mptr+M12) * ls_world.z ) >> W2V_SHIFT;
	ls_vector_view.z = ( *(mptr+M20) * ls_world.x
					   + *(mptr+M21) * ls_world.y
					   + *(mptr+M22) * ls_world.z ) >> W2V_SHIFT;
}

/**************************************************************************
 *		Position Global Light Vector In World Using Destination Position
 *				and Position of Light...
 *************************************************************************/

void	phd_PointLight( PHD_3DPOS *destpos, PHD_3DPOS *lightpos )
{
	PHD_ANGLE	angles[2];

	phd_GetVectorAngles( destpos->x_pos-lightpos->x_pos,  // Get Angles
						 destpos->y_pos-lightpos->y_pos,  // Of Light
						 destpos->z_pos-lightpos->z_pos,  // Vector
						 angles );
	phd_RotateLight( angles[1],angles[0] );
}
#endif

/*****************************************************************************
 *					Initialise 3D Window
 ****************************************************************************/
void	phd_InitWindow( int  view_angle )
{
//##	phd_scrwidth=512;
//##	phd_winwidth=phd_scrwidth;
//##	phd_winxmax=phd_winwidth-1;
//##	phd_centerx=phd_scrwidth>>1;
	GPU_SetHorizontalResolution(phd_scrwidth);

//	phd_winxmax = width-1;
//	phd_winymax = height-1;
//	phd_winwidth = width;							// These Are All
//	phd_winheight = height;                         // Macro definitions now
//	phd_centerx = width/2;
//	phd_centery = height/2;
//	phd_znear = nearz<<W2V_SHIFT;
//	phd_zfar = farz<<W2V_SHIFT;
//	phd_viewdist = farz;
	view_angle *= (182/2);              			// Halve angle and convert to 0-65535
	phd_persp = ( (phd_winwidth/2) * phd_cos(view_angle) ) / phd_sin(view_angle) ;
	phd_left = 0;
	phd_top = 0;
	phd_right = phd_winxmax;
	phd_bottom = phd_winymax;
	phd_mxptr = matrix_stack;    					// Initialise Matrix Pointer..
	SetGeomOffset( phd_centerx, phd_centery );		// Center Of Screen
	SetGeomScreen( phd_persp );                     // Perspective Divider..
}

/****************************************************************************
 *			Alter FOV for Cinematic stuff
 *		NOTE angle is in PHD_ANGLE format...
 ***************************************************************************/
void	AlterFOV( sint16 view_angle )
{
	view_angle /= 2;              			// Halve angle
	phd_persp = ( (phd_winwidth/2) * phd_cos(view_angle) ) / phd_sin(view_angle) ;
	SetGeomScreen( phd_persp );                     // Perspective Divider..
}


/****************************************************************************
 *					Push Current Matrix
 ***************************************************************************/
#if 0

void	phd_PushMatrix( void )
{
	sint32	*p1,*p2;

	p1 = phd_mxptr;
	phd_mxptr += 12;
	p2 = phd_mxptr;
	*(p2+M00) = *(p1+M00);
	*(p2+M01) = *(p1+M01);
	*(p2+M02) = *(p1+M02);
	*(p2+M03) = *(p1+M03);
	*(p2+M10) = *(p1+M10);
	*(p2+M11) = *(p1+M11);
	*(p2+M12) = *(p1+M12);
	*(p2+M13) = *(p1+M13);
	*(p2+M20) = *(p1+M20);
	*(p2+M21) = *(p1+M21);
	*(p2+M22) = *(p1+M22);
	*(p2+M23) = *(p1+M23);
}

/*****************************************************************************
 *				Make Current Matrix A Unit Matrix
 ****************************************************************************/
void	phd_UnitMatrix( void )
{
	sint32	*p1;

	p1 = phd_mxptr;
	*(p1+M00) = *(p1+M11) = *(p1+M22) = W2V_SCALE;
	*(p1+M01) = *(p1+M02) =
	*(p1+M10) = *(p1+M12) =
	*(p1+M20) = *(p1+M21) = 0;
}

/*****************************************************************************
 *				Push A Unit Matrix Onto Stack
 ****************************************************************************/
void	phd_PushUnitMatrix( void )
{
	phd_mxptr += 12;
	phd_UnitMatrix();
}

/*****************************************************************************
 *				Transpose Current Matrix
 ****************************************************************************/
void	phd_TransposeMatrix( void )
{
	sint32	*p1,swp;

	p1 = phd_mxptr;
	swp = *(p1+M01);
	*(p1+M01) = *(p1+M10);
	*(p1+M10) = swp;
	swp = *(p1+M12);
	*(p1+M12) = *(p1+M21);
	*(p1+M21) = swp;
	swp = *(p1+M20);
	*(p1+M20) = *(p1+M21);
	*(p1+M21) = swp;
}

/*******************************************************************************
 *			Calculate On screen Coordinates Of Bounding Box For Object.
 *
 * Returns 0 If Object Box totally OFF Screen
 *         1 If Object Box Totally ON Screen
 *        -1 If Object Box Is Z-Clipped Or X/Y Clipped
 ******************************************************************************/
//#define LEFT_TOLERANCE 100
//#define TOP_TOLERANCE 100
//#define RIGHT_TOLERANCE 200
//#define BOTTOM_TOLERANCE 200

//#define TOLERANCE 100
#define TOLERANCE 10

int		S_GetObjectBounds( sint16 *bptr )
{
	sint32	minx,maxx;
	sint32	miny,maxy;
	sint32	minz,maxz;
	sint32	zv,coord,numz,i;
	sint32	vertex[8][3];
	sint32	*vertptr,*mptr;


	mptr = phd_mxptr;
	if ( *(mptr+M23)>=phd_zfar )
		return(0);

	minx = (sint32)*(bptr+0);
	maxx = (sint32)*(bptr+1);
	miny = (sint32)*(bptr+2);
	maxy = (sint32)*(bptr+3);
	minz = (sint32)*(bptr+4);
	maxz = (sint32)*(bptr+5);

	vertex[0][0] = minx;		// V.0
	vertex[0][1] = miny;
	vertex[0][2] = minz;

	vertex[1][0] = maxx;		// V.1
	vertex[1][1] = miny;
	vertex[1][2] = minz;

	vertex[2][0] = maxx;		// V.2
	vertex[2][1] = maxy;
	vertex[2][2] = minz;

	vertex[3][0] = minx;		// V.3
	vertex[3][1] = maxy;
	vertex[3][2] = minz;

	vertex[4][0] = minx;		// V.4
	vertex[4][1] = miny;
	vertex[4][2] = maxz;

	vertex[5][0] = maxx;		// V.5
	vertex[5][1] = miny;
	vertex[5][2] = maxz;

	vertex[6][0] = maxx;		// V.6
	vertex[6][1] = maxy;
	vertex[6][2] = maxz;

	vertex[7][0] = minx;		// V.7
	vertex[7][1] = maxy;
	vertex[7][2] = maxz;


	numz = 0;
	minx = miny =  0x3fffffff;
	maxx = maxy = -0x3fffffff;
	vertptr = &vertex[0][0];
	for ( i=0; i<8; i++,vertptr+=3 )
	{
		zv = (sint32)(*(mptr+M20) * *(vertptr+0) + 				// First Work Out Z
			 		  *(mptr+M21) * *(vertptr+1) +              // view-frame Coordinate...
			 		  *(mptr+M22) * *(vertptr+2) +
			 		  *(mptr+M23) );

		if ( zv>phd_znear )
		{
			numz++;
			zv /= phd_persp;
			coord = (sint32)((*(mptr+M00) * *(vertptr+0) + 		// Calculate X coordinate....
					  		  *(mptr+M01) * *(vertptr+1) +
					  		  *(mptr+M02) * *(vertptr+2) +
					  		  *(mptr+M03) )/zv );
			if ( coord<minx )
				minx = coord;
			if ( coord>maxx )
				maxx = coord;
			coord = (sint32)((*(mptr+M10) * *(vertptr+0) + 		// Calculate Y coordinate....
					  		  *(mptr+M11) * *(vertptr+1) +
					  		  *(mptr+M12) * *(vertptr+2) +
					  		  *(mptr+M13) )/zv );
			if ( coord<miny )
				miny = coord;
			if ( coord>maxy )
				maxy = coord;
		}
	}
	minx += phd_centerx;							// Add on Screen Offsets
	maxx += phd_centerx;                            // to MinMax
	miny += phd_centery;
	maxy += phd_centery;

//	if ( !numz ||     								// If No Coords
//		 minx>phd_right || miny>phd_bottom ||       // Are On Screen Exit Now
//		 maxx<phd_left  || maxy<phd_top	)
//			return(0);

//	if ( numz<8 ||
//		 minx<(phd_left-LEFT_TOLERANCE)  || miny<(phd_top-TOP_TOLERANCE) ||			// Bounding Box is partially
//		 maxx>(phd_right+RIGHT_TOLERANCE) || maxy>(phd_bottom+BOTTOM_TOLERANCE))	// On screen or/and Z-Clipped...
//			return(-1);

	if ( !numz ||     								// If No Coords
		 minx>(phd_right+TOLERANCE) || miny>(phd_bottom+TOLERANCE) ||       // Are On Screen Exit Now
		 maxx<(phd_left-TOLERANCE)  || maxy<(phd_top-TOLERANCE)	)
			return(0);

	if ( numz<8 ||
		 minx<(phd_left-TOLERANCE)  || miny<(phd_top-TOLERANCE) ||			// Bounding Box is partially
		 maxx>(phd_right+TOLERANCE) || maxy>(phd_bottom+TOLERANCE))	// On screen or/and Z-Clipped...
			return(-1);

	return(1);										// Return 1 If Box Wholly On Screen...
}

#endif
